#!/usr/bin/python
# -*- coding: utf-8 -*-

# Hive Colony Framework
# Copyright (c) 2008-2015 Hive Solutions Lda.
#
# This file is part of Hive Colony Framework
#
# Hive Colony Framework is free software: you can redistribute it and/or modify
# it under the terms of the Apache License as published by the Apache
# Foundation, either version 2.0 of the License, or (at your option) any
# later version.
#
# Hive Colony Framework is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# Apache License for more details.
#
# You should have received a copy of the Apache License along with
# Hive Colony Framework If not, see <http://www.apache.org/licenses/>.

__author__ = "João Magalhães <joamag@hive.pt>"
""" The author(s) of the module """

__version__ = "1.0.0"
""" The version of the module """

__revision__ = "$LastChangedRevision$"
""" The revision number of the module """

__date__ = "$LastChangedDate$"
""" The last change date of the module """

__copyright__ = "Copyright (c) 2008-2015 Hive Solutions Lda."
""" The copyright for the module """

__license__ = "Apache License, Version 2.0"
""" The license for the module """

import colony

class ProtectionTest(colony.ColonyTestCase):
    """
    Test case class responsible for the management of the
    tests for the protection utilities.
    """

    def test_access(self):
        """
        Test that verifies the access to the public and
        the non possible access to the private methods.
        """

        mock = ProtectionMock()
        result = mock.public()
        self.assertEqual(result, "public")
        self.assert_raises(AttributeError, getattr, mock, "private")
        self.assert_raises(AttributeError, getattr, mock, "protected")

class ProtectionMock(colony.Protected):
    """
    Mock class to be used in the testing of protected
    attributes, should inherit from the (abstract) class
    responsible for the "protection".
    """

    name = "protected"
    """ The name attribute, should be protected by
    the protected infra-structure """

    @colony.public
    def public(self):
        return "public"

    def private(self):
        return "private"



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Hive Colony Framework
# Copyright (c) 2008-2015 Hive Solutions Lda.
#
# This file is part of Hive Colony Framework
#
# Hive Colony Framework is free software: you can redistribute it and/or modify
# it under the terms of the Apache License as published by the Apache
# Foundation, either version 2.0 of the License, or (at your option) any
# later version.
#
# Hive Colony Framework is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# Apache License for more details.
#
# You should have received a copy of the Apache License along with
# Hive Colony Framework If not, see <http://www.apache.org/licenses/>.

__author__ = "João Magalhães <joamag@hive.pt> & Tiago Silva <tsilva@hive.pt>"
""" The author(s) of the module """

__version__ = "1.0.0"
""" The version of the module """

__revision__ = "$LastChangedRevision$"
""" The revision number of the module """

__date__ = "$LastChangedDate$"
""" The last change date of the module """

__copyright__ = "Copyright (c) 2008-2015 Hive Solutions Lda."
""" The copyright for the module """

__license__ = "Apache License, Version 2.0"
""" The license for the module """

prefix_paths = {
    "default" : {
    }
}
""" The prefix path maps, contains a series of associated
values, should be used with care (mostly maintained for legacy reasons) """

plugin_path_list = [
    "plugins",
    "plugins/*plugin"
]
""" The list of plugin paths, these directories are going
to be searched for plugin files (via criteria) and the
plugins that exist there are going to be loaded """

library_path_list = [
    "libraries"
]
""" The list of library paths, these paths are going to
be exported to the global system path and may be used
independently of the execution location """

meta_path_list = [
    "meta",
    "meta/*config"
]
""" The list of meta paths, that should contain configuration
directory structures, the runtime loaded properties will
depend of these values """

logger_path = "log"
""" The path to the logger directory, this should be a path
relative to the current execution manager path, if this path
does not exists it may be created at runtime """

level = "WARNING"
""" The verbosity level that is going to be used as part of
the colony logging infra-structure, this value will affect all
of the currently attached loggers """

stop_on_cycle_error = False
""" If the plugin loading system should stop whenever an exception
is raised as part of the loading cycle (change with case) """

daemon_file_path = None
""" The path to the daemon (pid) file that is going to be used by
the executing colony instance, should only be used for detached processes"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Hive Colony Framework
# Copyright (c) 2008-2015 Hive Solutions Lda.
#
# This file is part of Hive Colony Framework
#
# Hive Colony Framework is free software: you can redistribute it and/or modify
# it under the terms of the Apache License as published by the Apache
# Foundation, either version 2.0 of the License, or (at your option) any
# later version.
#
# Hive Colony Framework is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# Apache License for more details.
#
# You should have received a copy of the Apache License along with
# Hive Colony Framework If not, see <http://www.apache.org/licenses/>.

__author__ = "João Magalhães <joamag@hive.pt>"
""" The author(s) of the module """

__version__ = "1.0.0"
""" The version of the module """

__revision__ = "$LastChangedRevision$"
""" The revision number of the module """

__date__ = "$LastChangedDate$"
""" The last change date of the module """

__copyright__ = "Copyright (c) 2008-2015 Hive Solutions Lda."
""" The copyright for the module """

__license__ = "Apache License, Version 2.0"
""" The license for the module """

import os
import sys
import time

from . import legacy

CPYTHON_ENVIRONMENT = "cpython"
""" CPython environment value """

JYTHON_ENVIRONMENT = "jython"
""" Jython environment value """

IRON_PYTHON_ENVIRONMENT = "iron_python"
""" IronPython environment value """

WINDOWS_OS = "windows"
""" The windows os value """

MAC_OS = "mac"
""" The mac os value """

UNIX_OS = "unix"
""" The unix os value """

OTHER_OS = "other"
""" The other os value """

UID_PRECISION = 8
""" Unique id precision """

class WaitInput(object):
    """
    Wait input file used to overcome the problem
    with being stuck in the console input.
    This file like object reads something as a virtual
    timeout object avoid the proper read and unblocking
    the current in/out operation.
    """

    def readline(self):
        """
        Reads a "line" from the wait input.
        This is an empty read as all it does
        is wait for a while and then return.
        """

        # sleeps for a little bit
        time.sleep(1.0)

        # returns an empty (wait string)
        return ""

class QueueEvent(object):
    """
    The class that describes an event to be
    used in a generic event queue.
    """

    event_name = None
    """ The name of the event """

    event_args = []
    """ The arguments of the event """

    def __init__(self, event_name, event_args = []):
        """
        Constructor of the class.

        @type event_name: String
        @param event_name: The name of the event.
        @type event_args: List
        @param event_args: The arguments of the event.
        """

        self.event_name = event_name
        self.event_args = event_args

class Plugins(object):
    """
    Class used as storage for the various plugin
    instance references indexed by their name.
    """

    pass

def module_import(module_name):
    """
    Imports the module with the given name, this import
    operation is recursive meaning that inner packages
    are also going to be imported.

    @type module_name: String
    @param module_name: The name of the module to be imported,
    this value may contain multiple "sub" packages.
    @rtype: module
    @return: The imported module as a variable reference.
    """

    module = __import__(module_name)
    components = module_name.split(".")
    for component in components[1:]:
        module = getattr(module, component)
    return module

def resolve_manager(exec_path = None):
    """
    Master resolver for the manager path, it's responsible
    for the decision to use the current and possible master
    directory structure or the personal (home directory based)
    strategy that is used as fallback.

    Note that the master structure should only be used for
    development purposes.

    @type exec_path: String
    @param exec_path: The currently executing path, should be
    retrieved from the first file to be executed in the call
    stack, this is the path to be inspected for master.
    @rtype: String
    @return: The final "resolved" manager path after the proper
    validation operations have been done.
    """

    is_venv = hasattr(sys, "real_prefix")
    personal_path = sys.prefix if is_venv else os.path.expanduser("~")
    personal_path = os.path.join(personal_path, ".colony")
    master_path = os.environ.get("COLONY_HOME", exec_path)
    manager_path = master_path if is_master(master_path) else personal_path
    manager_path = os.path.abspath(manager_path)
    manager_path = os.path.normpath(manager_path)
    return manager_path

def ensure_tree(path):
    """
    Ensures that the proper colony plugin system directory
    structure is created under the provided path.

    Note that the top level directory will be created in case
    it does not exists, the permissions used for such operation
    will be the default ones (may be not secure enough).

    @type path: String
    @param path: The file path of the root plugin system directory
    for which the structure is going to be created.
    """

    is_personal = not is_master(path)
    config_path = os.path.join(path, "config")
    containers_path = os.path.join(path, "containers")
    deploy_path = os.path.join(path, "deploy")
    libraries_path = os.path.join(path, "libraries")
    log_path = os.path.join(path, "log")
    meta_path = os.path.join(path, "meta")
    plugins_path = os.path.join(path, "plugins")
    scripts_path = os.path.join(path, "scripts")
    tmp_path = os.path.join(path, "tmp")
    var_path = os.path.join(path, "var")
    env_path = os.path.join(path, "colony.json")
    if not os.path.exists(path): os.makedirs(path)
    if not os.path.exists(config_path): os.makedirs(config_path)
    if not os.path.exists(containers_path): os.makedirs(containers_path)
    if not os.path.exists(deploy_path): os.makedirs(deploy_path)
    if not os.path.exists(libraries_path): os.makedirs(libraries_path)
    if not os.path.exists(log_path): os.makedirs(log_path)
    if not os.path.exists(meta_path): os.makedirs(meta_path)
    if not os.path.exists(plugins_path): os.makedirs(plugins_path)
    if not os.path.exists(scripts_path): os.makedirs(scripts_path)
    if not os.path.exists(tmp_path): os.makedirs(tmp_path)
    if not os.path.exists(var_path): os.makedirs(var_path)
    if not os.path.exists(env_path) and is_personal: open(env_path, "a").close()

def is_master(path):
    """
    Verifies if the provided file path is considered to be the
    root path of a colony master directory structure. A colony
    master directory structure is considered to be a directory
    structure where the colony package system and the other
    directories mix together (typically used for development).

    This structure is the opposite of the personal strategy where
    the colony instance is stored separated to the logic of colony
    typically used for production environments.

    @type path: String
    @param path: The file path that is going to be verified to
    be part of a colony master directory structure.
    @rtype: bool
    @return: If the provided path is considered to be a normalized
    master directory structure for colony.
    """

    if path == None: return False
    package_path = os.path.join(path, "colony")
    plugins_path = os.path.join(path, "plugins")
    return os.path.isdir(package_path) and os.path.isdir(plugins_path)

def get_environment():
    """
    Retrieves the current python environment, there
    should be multiple environments depending on the
    kind of python interpreter being used.

    @rtype: String
    @return: The type of the current python environment,
    taking into account the current interpreter information.
    """

    platform = sys.platform
    if not platform.find("java") == -1: return JYTHON_ENVIRONMENT
    elif not platform.find("cli") == -1: return IRON_PYTHON_ENVIRONMENT
    else: return CPYTHON_ENVIRONMENT

def get_operative_system():
    """
    Retrieves the current operative system, this is a
    normalized operation that uses the underlying python
    infra-structure and normalizes the result.

    @rtype: String
    @return: The type of the current operative system.
    """

    os_name = os.name
    if os_name == "nt" or os_name == "dos": return WINDOWS_OS
    elif os_name == "mac": return MAC_OS
    elif os_name == "posix": return UNIX_OS
    return OTHER_OS

def get_timestamp_uid():
    """
    Retrieves a unique id based in the current timestamp.
    This value should not be used for high accuracy or
    cryptographic operations as it's not very unique and
    it is not entropy safe.

    @rtype: String
    @return: A unique id based in the current timestamp.
    """

    timestamp = time.time()
    float_value = timestamp * (10 ** UID_PRECISION)
    integer_value = legacy.LONG(float_value)
    string_value = str(integer_value)
    return string_value



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Hive Colony Framework
# Copyright (c) 2008-2015 Hive Solutions Lda.
#
# This file is part of Hive Colony Framework
#
# Hive Colony Framework is free software: you can redistribute it and/or modify
# it under the terms of the Apache License as published by the Apache
# Foundation, either version 2.0 of the License, or (at your option) any
# later version.
#
# Hive Colony Framework is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# Apache License for more details.
#
# You should have received a copy of the Apache License along with
# Hive Colony Framework If not, see <http://www.apache.org/licenses/>.

__author__ = "João Magalhães <joamag@hive.pt>"
""" The author(s) of the module """

__version__ = "1.0.0"
""" The version of the module """

__revision__ = "$LastChangedRevision$"
""" The revision number of the module """

__date__ = "$LastChangedDate: 2011-01-15 17:29:58 +0000 (sÃ¡b, 15 Jan 2011) $"
""" The last change date of the module """

__copyright__ = "Copyright (c) 2008-2015 Hive Solutions Lda."
""" The copyright for the module """

__license__ = "Apache License, Version 2.0"
""" The license for the module """

PUBLIC_VALUE = "__public__"
""" The public value """

def public(function):
    """
    Decorator used to assign the public attribute to methods.

    @type function: Function
    @param function: The function to sets the public visibility.
    @rtype: Function
    @return: The sent function (returned as a normal decorator).
    """

    # sets the (decorated) function as public and then returns
    # the function to the caller (default decorator behavior)
    function.__public__ = True
    return function

class Protected(object):
    """
    Base class of all classes that want to hide protected
    attributes from public access.
    This class should be carefully used as it may
    create unexpected side effects.
    """

    def __new__(cls, *args, **kwargs):
        # creates a new object reference, note that
        # the object is created without arguments,
        # avoiding possible runtime errors
        object_reference = object.__new__(cls)

        # initializes the class reference with the new
        # object reference and the arguments
        cls.__init__(object_reference, *args, **kwargs)

        def __getattr__(self, name):
            # retrieves the attribute from the object reference
            attribute = getattr(object_reference, name)

            # in case the attribute is public
            if hasattr(attribute, PUBLIC_VALUE):
                # returns the attribute (valid)
                return attribute
            # in case the class is public
            elif hasattr(cls, PUBLIC_VALUE):
                # in case the name is in the public
                # attributes list
                if name in cls.__public__:
                    # returns the attribute (valid)
                    return attribute

            # raises the attribute error, meaning that the attribute
            # exits but it's not exposed as a public attribute
            raise AttributeError(
                "attribute '%s' of class '%s' is not public" %\
                (name, cls.__name__)
            )

        def __setattr__(self, name, value):
            cls.__setattr__(self, name, value)

        def is_own_magic(cls, name, without = []):
            """
            Checks if the given name is a magic attribute
            in the class.

            @type name: String
            @param name: The name to be tested for "magic".
            @type without: List
            @param without: The list of attributes to exclude magic.
            @rtype: bool
            @return: The result of the "magic" test.
            """

            # in case the name is magic and is not in the without list
            return name not in without and name.startswith("__") and name.endswith("__")

        # creates a new proxy instance with the protected name
        Proxy = type("Protected(%s)" % cls.__name__, (), {})

        # retrieves the class names
        class_names = dir(cls)

        # retrieves the proxy names
        proxy_names = dir(Proxy)

        # iterates over all the class names
        for class_name in class_names:
            # tests to check if the name is magic
            if not is_own_magic(cls, class_name, proxy_names):
                # continues the loop
                continue

            try:
                # retrieves the attribute
                attribute = getattr(object_reference, class_name)

                # sets the attribute in the proxy
                setattr(Proxy, class_name, attribute)
            except TypeError:
                pass

        # sets the new retrieve attribute and
        # set attribute references
        Proxy.__getattr__ = __getattr__
        Proxy.__setattr__ = __setattr__

        # creates a new proxy instance
        proxy_instance = Proxy()

        # sets the proxy instance in the object reference
        object_reference._proxy_instance = proxy_instance

        # returns the proxy instance
        return proxy_instance



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Hive Colony Framework
# Copyright (c) 2008-2015 Hive Solutions Lda.
#
# This file is part of Hive Colony Framework
#
# Hive Colony Framework is free software: you can redistribute it and/or modify
# it under the terms of the Apache License as published by the Apache
# Foundation, either version 2.0 of the License, or (at your option) any
# later version.
#
# Hive Colony Framework is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# Apache License for more details.
#
# You should have received a copy of the Apache License along with
# Hive Colony Framework If not, see <http://www.apache.org/licenses/>.

__author__ = "João Magalhães <joamag@hive.pt>"
""" The author(s) of the module """

__version__ = "1.0.0"
""" The version of the module """

__revision__ = "$LastChangedRevision$"
""" The revision number of the module """

__date__ = "$LastChangedDate: 2011-01-15 17:29:58 +0000 (sÃ¡b, 15 Jan 2011) $"
""" The last change date of the module """

__copyright__ = "Copyright (c) 2008-2015 Hive Solutions Lda."
""" The copyright for the module """

__license__ = "Apache License, Version 2.0"
""" The license for the module """

import sys
import math

FLOAT_PRECISION = 14
""" The amount of precision (in decimal places) that
is going to be used in the calculus of the delta """

DELTA = 1 / math.pow(10, FLOAT_PRECISION)
""" The delta value that is going to be applied to
the round operation representing the old strategy of
rounding, this is required so that a proper half way
up strategy is applied in the rounding """

QUANTIFIERS = {}
""" The map of quantifier strings indexed by
the number of decimal places for their round """

_round = round

def roundi(value, places, precise = False):
    """
    Rounds the provided float value to the provided
    number of decimal places returning a floating
    point number representing the result.

    The provided value is rounded using the round half up
    strategy where (to nearest with ties going away from zero).

    The rounding procedure is done using the "old" rounding
    method, that contains a floating point based error.
    This is the rounding method expected by the current
    colony infra-structure and as such it should be applied
    at the beginning of a new plugin system instance.

    This rounding method incurs in major performance issues
    and should only be used for interpreters that use the
    "new" rounding method.

    @type value: float
    @param value: The floating point value to be rounded
    @type places: int
    @param places: The number of decimal places to be used
    in the rounding operation.
    @type precise: bool
    @param precise: If the precise mode should be used where
    the delta value is calculated taking into account the
    number of places of the provided float, otherwise the
    "safe" delta is used, which should be good for most of
    the float number to be used.
    @rtype: float
    @return: The resulting rounded value according to the
    round half up strategy.
    @see: http://docs.python.org/2/tutorial/floatingpoint.html
    """

    delta = _delta(value) if precise else DELTA
    return _round_t(value + delta, places)

def rounds(value, places):
    """
    Considered to be the "safest" way of rounding a number either
    a "native" float number or a decimal, this operation should
    always be used whenever the rounding result should be accurate.

    For the decimal data type usage this is considered to be the
    only way of ensuring proper and deterministic rounding.

    Note that this is an expensive operation and proper care should
    be taken to avoid extra processor usage.

    @type value: float
    @param value: The floating point value to be rounded
    @type places: int
    @param places: The number of decimal places to be used
    in the rounding operation.
    @rtype: float
    @return: The resulting rounded value according to the
    current (most precise) way of rounding.
    """

    return roundi(value, places, precise = True)

def roundt(value, places):
    """
    Simple rounding utility function that performs the currently
    selected rounding operation on the provided value and then
    re-casts the resulting value back to the original data type.

    This function is relevant/useful for situation where the base
    value inherits indirectly from the float class.

    @type value: float
    @param value: The value that is meant to be rounded and then
    "casted" back to the original data type.
    @param places: The number of decimal places to be used
    in the rounding operation.
    @rtype: float
    @return: The resulting rounded value according to the
    default rounding strategy defined.
    """

    value_t = type(value)
    result = round(value, places)
    return result if type(result) == value_t else value_t(result)

def round_apply(force = False):
    """
    Applies the "old" rounding strategy to the current
    interpreted in a global fashion (override).

    This method only applies the rounding method for
    interpreters that uses the new rounding method
    avoiding the apply of the calculus for old rounding
    method interpreters (provides performance).

    @type force: bool
    @param force: If the apply operation should be performed
    for environments where it's not required (old rounding).
    """

    # verifies that the current executing version of
    # the interpreter is using the new rounding methods
    # in case it's not no apply occurs (not required)
    new_round = round_is_new()
    if not new_round and not force: return

    # updates the built-in round function with the new
    # round function so that the rounds are coherent, note
    # that the builtins reference may be either map based
    # or module based, logic must take care of both cases
    builtins = globals()["__builtins__"]
    if type(builtins) == dict: builtins["round"] = roundi
    else: builtins.round = roundi

def round_unapply(force = False):
    """
    Reverts the apply operation of the "old" rounding
    strategy back to the original built-in rounding.

    This method should be used carefully as it may produce
    some unexpected results.

    @type force: bool
    @param force: If the unapply operation should be performed
    for environments where it's not required (old rounding).
    """

    # verifies that the current executing version of
    # the interpreter is using the new rounding methods
    # in case it's not no unapply occurs (not required)
    new_round = round_is_new()
    if not new_round and not force: return

    # updates the built-in round function with the old
    # round function so that the rounds are reverted, note
    # that the builtins reference may be either map based
    # or module based, logic must take care of both cases
    builtins = globals()["__builtins__"]
    if type(builtins) == dict: builtins["round"] = _round_t
    else: builtins.round = _round_t

def round_is_new():
    """
    Verifies that the current execution version of the interpreter
    is running the new rounding strategy, meaning that the round
    operations will be performed using the half way undefined strategy.

    This method infers the strategy that is currently in use for
    rounding based on the version of the interpreted.

    @rtype: bool
    @return: If the current (python) interpreter is running the
    new rounding strategy (half way undefined).
    @see: http://docs.python.org/2/tutorial/floatingpoint.html
    """

    # unpacks the system's version information tuple
    # into its major and minor values to be used for
    # the checking of the new round method
    major = sys.version_info[0]
    minor = sys.version_info[1]

    # verifies that the current executing version of
    # the interpreter is using the new rounding methods
    # in case it's not no unapply occurs (not required)
    new_round = major > 3 or (major == 3 and minor >= 1) or\
        (major == 2 and minor >= 7)
    return new_round

def _round_t(value, places):
    """
    Internal function similar to the the type one but uses
    the internal (built-in) version of the rounder to perform
    the rounding operation.

    This function should only be used for old python 2 based
    versions and not for the new python 3+ versions where the
    proper overriding of the __round__ method is preferred.

    @type value: float
    @param value: The value that is meant to be rounded and then
    "casted" back to the original data type.
    @param places: The number of decimal places to be used
    in the rounding operation.
    @rtype: float
    @return: The resulting rounded value according to the
    default rounding strategy defined.
    """

    value_t = type(value)
    result = _round(value, places)
    return result if type(result) == value_t else value_t(result)

def _delta(value):
    """
    Calculates the proper minimum delta value that may be
    applied to the provided value so that it's still able
    to represent the "next" value.

    This is an expensive operation operation and should be
    used carefully to avoid extra computation.

    @type value: float
    @param value: The value for which the proper (minimum)
    delta value is going to be calculated.
    @rtype: float
    @param float: The (minimum) delta value calculated for
    the provided float value, this value may be used for
    proper "old" rounding strategy.
    """

    integer = abs(int(value // 1))
    count = 1 if integer == 0 else int(math.log10(integer)) + 1
    places = FLOAT_PRECISION - count
    if places < 1: places = 1
    delta = 1 / math.pow(10, places)
    return delta

# verifies if the current interpreter version is python 3+ and
# if that's the case used the builtin round function instead to
# allow data type casting through the __round__ magic method
if sys.version_info[0] >= 3: _round_t = _round

# updates the round reference in the builtin dictionary so that
# the proper type based casting is used instead of the builtin
# functions (allows proper data type casting for python 2)
builtins = globals()["__builtins__"]
if type(builtins) == dict: builtins["round"] = _round_t
else: builtins.round = _round_t



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Hive Colony Framework
# Copyright (c) 2008-2015 Hive Solutions Lda.
#
# This file is part of Hive Colony Framework
#
# Hive Colony Framework is free software: you can redistribute it and/or modify
# it under the terms of the Apache License as published by the Apache
# Foundation, either version 2.0 of the License, or (at your option) any
# later version.
#
# Hive Colony Framework is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# Apache License for more details.
#
# You should have received a copy of the Apache License along with
# Hive Colony Framework If not, see <http://www.apache.org/licenses/>.

__version__ = "1.0.0"
""" The version of the module """

__revision__ = "$LastChangedRevision$"
""" The revision number of the module """

__date__ = "$LastChangedDate$"
""" The last change date of the module """

__copyright__ = "Copyright (c) 2008-2015 Hive Solutions Lda."
""" The copyright for the module """

__license__ = "Apache License, Version 2.0"
""" The license for the module """

from . import config
from . import decorators
from . import exceptions
from . import information
from . import legacy
from . import loggers
from . import system
from . import test
from . import util

from .config import conf, conf_prefix, conf_s
from .decorators import load_plugin, plugin_meta_information, load_allowed, load_allowed_capability,\
    unload_allowed, unload_allowed_capability, inject_dependencies, plugin_inject, event_handler,\
    event_handler_method, set_configuration_property, set_configuration_property_method,\
    unset_configuration_property, unset_configuration_property_method, plugin_call, create_load_plugin_interceptor
from .exceptions import ColonyException, PluginSystemException, PluginClassNotAvailable, InvalidCommand,\
    InvalidArgument, OperationNotComplete
from .information import VERSION, RELEASE, BUILD, RELEASE_DATE, RELEASE_DATE_TIME, ENVIRONMENT_VERSION,\
    ENVIRONMENT, DEFAULT_ENCODING, DATE_FORMAT, DATE_TIME_FORMAT, INFORMATION_PATH
from .loggers import BroadcastHandler, MemoryHandler
from .system import System, Plugin, PluginManagerPlugin, PluginManager, Dependency, PluginDependency,\
    PackageDependency, Condition, OperativeSystemCondition, Capability, Event, PluginThread,\
    PluginEventThread
from .test import Test
from .util import CPYTHON_ENVIRONMENT, JYTHON_ENVIRONMENT, IRON_PYTHON_ENVIRONMENT, WINDOWS_OS, MAC_OS,\
    UNIX_OS, OTHER_OS, UID_PRECISION, WaitInput, QueueEvent, Plugins, module_import, resolve_manager,\
    ensure_tree, is_master, get_environment, get_operative_system, get_timestamp_uid



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Hive Colony Framework
# Copyright (c) 2008-2015 Hive Solutions Lda.
#
# This file is part of Hive Colony Framework
#
# Hive Colony Framework is free software: you can redistribute it and/or modify
# it under the terms of the Apache License as published by the Apache
# Foundation, either version 2.0 of the License, or (at your option) any
# later version.
#
# Hive Colony Framework is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# Apache License for more details.
#
# You should have received a copy of the Apache License along with
# Hive Colony Framework If not, see <http://www.apache.org/licenses/>.

__author__ = "João Magalhães <joamag@hive.pt>"
""" The author(s) of the module """

__version__ = "1.0.0"
""" The version of the module """

__revision__ = "$LastChangedRevision$"
""" The revision number of the module """

__date__ = "$LastChangedDate$"
""" The last change date of the module """

__copyright__ = "Copyright (c) 2008-2015 Hive Solutions Lda."
""" The copyright for the module """

__license__ = "Apache License, Version 2.0"
""" The license for the module """

import re
import os
import sys
import glob
import shutil
import zipfile
import tempfile

import colony

INDENT = 0
""" The global reference to the indentation level that
is going to be printed at the start of each output
operation, this value should be used carefully to avoid
any thread related problems (not thread safe) """

DEFAULT_TARGET = "colony"
""" The default directory to be used as target in
case no target path is provided (default name) """

DEFAULT_ROOT = "COLONY_ROOT"
""" The default name for the file to be used to
indicate the root directory of a colony instance """

PACK_FILE = "colony.zip"
""" The name of the file that will be the packing
reference of the instance """

EXTENSIONS = dict(
    bundle = ".cbx",
    plugin = ".cpx",
    config = ".ccx"
)
""" The map associating the various types of
colony packages with the associated extension """

REMOVALS = (
    "colony.egg-info",
    "EGG-INFO"
)
""" The list of paths to be removed because there's
no use for them in the target colony instance """

REPO_URL = "https://colony.bemisc.com/"
""" The basic default open repository for colony packages
this is always going to be used together with the current
user wide configuration values """

STAGE_URL = "https://colony.stage.hive.pt/"
""" The test url version of the colony repository, used
for the downloading of staging ready only packages """

def resolve_manager(path, ensure = True):
    manager_path = colony.resolve_manager(path)
    if ensure: colony.ensure_tree(manager_path)
    return manager_path

def output(message, flush = True):
    print((" " * INDENT) + message)
    if flush: sys.stdout.flush()

def indent():
    global INDENT
    INDENT += 1

def unindent():
    global INDENT
    INDENT -= 1

def help():
    output("CPM - package management for Colony Framework")
    print("")
    print("  cpm clone <target>           Clones the base colony instance into the target directory (new project)")
    print("  cpm cleanup <target>         Cleans the current instance removing extra files")
    print("  cpm pack <target>            Packs the current instance into a .zip file")
    print("  cpm generate [target] <...>  Generates a .json descriptor file for the provided python "\
        "file and then runs the build operation for the generated .json file, effectively build the package item"
    )
    print("  cpm build [descriptor] <...> Builds the target .json descriptor file into a package file")
    print("  cpm deploy [package]         Deploys the target .cbx ile into the current instance")
    print("  cpm info [package]           Prints information about the package to the standard output")
    print("  cpm install [name] <...>     Installs the package with the provided name from the remote repositories")
    print("  cpm upgrade                  Updates the complete set of packages deployed in the instance")
    print("  cpm require [path] <...>     Installs the complete set of packages defined in the requirements file")
    print("  cpm upload [target] <repo>   Generates a package for the provided path and then uploads it "\
        "to the currently configured primary repository, or another repository if defined"
    )

def version():
    output("CPM - package management for Colony Framework")

def env():
    # retrieves the current working directory (cwd)
    # in order to be used in as fallback case
    cwd = os.getcwd()

    # retrieves the complete set of information that is
    # going to be printed as part of the info printing
    manager_path = resolve_manager(cwd)

    # prints the complete set of information to the user
    # so that it may take some decisions on the interaction
    output("manager_path := %s" % manager_path)

def clone():
    # in case there are enough arguments for the
    # deduction of the target path uses the provided
    # parameters otherwise used the default name
    # for the target path
    if len(sys.argv) > 2: target = sys.argv[2]
    else: target = DEFAULT_TARGET

    # retrieves the complete (and normalized) colony
    # path and then uses it to create the new instance
    # (cloned from the current instance)
    colony_path = os.path.normpath(os.path.dirname(__file__))
    shutil.copytree(colony_path, target)

    # iterates over all the paths to be removed from
    # the newly creates instance (not required anymore)
    for path in REMOVALS:
        # joins the path for the removal with the target
        # path and then in case it exists removes it
        _path = os.path.join(target, path)
        if not os.path.exists(_path): continue
        shutil.rmtree(_path)

    # runs the cleanup process on the target path
    # so that unnecessary files are removed
    _cleanup(target)

    # opens the colony instance reference file (this
    # file indicates the root of the colony instance)
    root_file_path = os.path.join(target, DEFAULT_ROOT)
    try: root_file = open(root_file_path, "a")
    finally: root_file.close()

def cleanup():
    """
    Cleans the target colony instance removing all the
    non mandatory files from the various internal directories.

    The removed files include python compiled files, extra
    directories, etc.

    The strategy used is conservative so whenever in doubt
    the process will not remove the file.
    """

    # retrieves the current working directory (cwd)
    # in order to be used in as fallback case
    cwd = os.getcwd()

    # in case there are enough arguments for the
    # deduction of the target path uses the provided
    # parameters otherwise used the default name
    # for the target path
    if len(sys.argv) > 2: target = sys.argv[2]
    else: target = resolve_manager(cwd)

    # in case not target was expanded the current directory
    # is used (assumes) the administration file is stored
    # at the same location as the colony instance
    target = target or os.path.normpath(os.path.dirname(__file__))

    # in case not target path is defined must raise
    # a runtime error
    if not target: raise RuntimeError("no instance found")

    # runs the cleanup command on the target path
    # so that all the non required files are removed
    _cleanup(target)

def pack():
    # retrieves the current working directory (cwd)
    # in order to be used in as fallback case
    cwd = os.getcwd()

    # in case there are enough arguments for the
    # deduction of the target path uses the provided
    # parameters otherwise used the default name
    # for the target path
    if len(sys.argv) > 2: target = sys.argv[2]
    else: target = resolve_manager(cwd)

    # in case not target was expanded the current directory
    # is used (assumes) the administration file is stored
    # at the same location as the colony instance
    target = target or os.path.normpath(os.path.dirname(__file__))

    # in case not target path is defined must raise
    # a runtime error, because it's not possible to proceed
    if not target: raise RuntimeError("no instance found")

    # runs the pack command on the target path
    # to create the packed file for the colony instance
    _pack(target)

def generate():
    # in case there're not enough arguments to be
    # able to retrieve the specification file raises
    # a runtime error
    if len(sys.argv) < 3: raise RuntimeError("no plugin file provided")

    # retrieves the target plugin file paths and uses them
    # for the generation of the descriptor file that should
    # represent the same plugin in terms of meat information
    targets = sys.argv[2:]
    for target in targets: _generate(target)

def build():
    # in case there're not enough arguments to be
    # able to retrieve the specification file raises
    # a runtime error
    if len(sys.argv) < 3: raise RuntimeError("no descriptor provided")

    # retrieves the descriptor file from the arguments and
    # uses it to run the build structure
    descriptors = sys.argv[2:]
    for descriptor in descriptors: _build(descriptor)

def deploy():
    # in case there're not enough arguments to be
    # able to retrieve the specification file raises
    # a runtime error, the name of the file is required
    if len(sys.argv) < 3: raise RuntimeError("no package provided")

    # retrieves the package files from the arguments and
    # uses them to run the deploy the packages into the currently
    # defined global instance (default colony instance)
    packages = sys.argv[2:]
    for package in packages: _deploy(package)

def install():
    # in case there're not enough arguments to be
    # able to retrieve the package file for install
    if len(sys.argv) < 3: raise RuntimeError("no name of package provided")

    # runs the install operation using the provided names
    # as reference, note that these names may contain an
    # optional version value attached to them
    names = sys.argv[2:]
    for name in names: _install(name)

def require():
    # in case there're not enough arguments to be
    # able to retrieve the requirements file
    if len(sys.argv) < 3: raise RuntimeError("no requirements file provided")

    # runs the requirements operation with the provided
    # paths, this is a recursive operation and may take
    # a while to reach the complete and final state
    paths = sys.argv[2:]
    for path in paths: _require(path)

def upgrade():
    _upgrade()

def upload():
    # in case there're not enough arguments to be
    # able to retrieve the specification file raises
    # a runtime error, the name of the file is required
    if len(sys.argv) < 3: raise RuntimeError("no plugin file provided")

    # in case there's an extra argument provided it's assumed
    # that it's the name of the target repo
    if len(sys.argv) > 3: repo = sys.argv[3]
    else: repo = "colony"

    # runs the upload operation for the target path, this
    # should be a base file to be packed and then uploaded
    # to the currently defined repository
    target = sys.argv[2]
    _upload(target, repo = repo)

def info():
    # in case there're not enough arguments to be
    # able to retrieve the specification file raises
    # a runtime error, the name of the file is required
    if len(sys.argv) < 3: raise RuntimeError("no package provided")

    # retrieves the package file from the arguments and
    # uses it to runs the information command to be able
    # to print some information about the package
    package = sys.argv[2]
    _info(package)

def _cleanup(path, empty_extra = True):
    # retrieves the path to the series of sub
    # directories to be "cleaned"
    log_path = os.path.join(path, "log")
    meta_path = os.path.join(path, "meta")

    # removes the files using extension based rules
    # on the defined directories, this should normalize
    # the directory structure and avoid problems
    _cleanup_files(path, re.compile(".*\.pyc$"))
    _cleanup_files(log_path, re.compile(".*\.log$"))
    _cleanup_files(log_path, re.compile(".*\.log.[0-9]+$"))
    _cleanup_files(log_path, re.compile(".*\.err$"))
    _cleanup_files(log_path, re.compile(".*\.err.[0-9]+$"))
    empty_extra and _cleanup_directories(meta_path, re.compile(""))

def _cleanup_directories(path, extension):
    # verifies that the provided path exists and is
    # a valid directory, in case it does not returns
    # immediately to avoid any problem
    if not os.path.isdir(path): return

    # lists all the entries in the provided path
    # in order to filter the ones to be removed
    entries = os.listdir(path)

    # iterates over all the entries to check the ones
    # that correspond to directories and run the cleanup
    # on each of their files
    for entry in entries:
        _path = os.path.join(path, entry)
        if not os.path.isdir(_path): continue
        _cleanup_files(_path, extension)

def _cleanup_files(path, extension):
    # verifies that the provided path exists and is
    # a valid directory, in case it does not returns
    # immediately to avoid any problem
    if not os.path.isdir(path): return

    # lists all the entries in the provided path
    # in order to filter the ones to be removed
    entries = os.listdir(path)

    # iterates over all the entries to treat them
    # in case it's necessary
    for entry in entries:
        # joins the path with the entry to create
        # the complete entry path, then runs the
        # appropriate iteration loop operations
        _path = os.path.join(path, entry)
        if os.path.isdir(_path): _cleanup_files(_path, extension); continue
        if extension.match(_path): os.remove(_path)

    # verifies that the (directory) path exists otherwise
    # returns immediately then lists the directory to check
    # if there are still files contained in it and in case
    # there is returns immediately, then proceeds with the
    # directory removal for the current path
    if not os.path.exists(path): return
    if os.listdir(path): return
    os.rmdir(path)

def _pack(path):
    # runs the cleanup process for the provided path
    # in order to prepare the current structure for
    # the packing of the files
    _cleanup(path)

    # joins the current path with the top level reference
    # path and then joins the pack file name to it
    _path = os.path.join(path, "..")
    archive_path = os.path.join(_path, PACK_FILE)
    archive_path = os.path.normpath(archive_path)

    # opens the archive path as a zip file for writing and
    # then writes the current "instance" directory into the zip
    file = zipfile.ZipFile(archive_path, "w", zipfile.ZIP_DEFLATED)
    try: _zip_directory(path, "/", file)
    finally: file.close()

    # prints a message about the packing operation that has just
    # been performed on the current running colony instance
    output("Packed %s into %s" % (path, archive_path))

def _generate(path, build = True, delete = True):
    # normalizes the path so that the value that is going to be
    # used from no on is going to be the correct one according
    # to the current operative system specifications
    path = os.path.abspath(path)
    path = os.path.normpath(path)

    # verifies the type of generation that is going to be performed, taking into
    # account some of the characteristics of the provided path
    is_plugin = path.endswith(".py")
    is_config = os.path.isdir(path) and ".project" in os.listdir(path)

    # verifies the type of generation that is meant to be performed and runs it,
    # in case no valid type is found for the provided path an exception is raised
    # indicating the problem (as expected by the current specification)
    if is_plugin: descriptor_path = _generate_plugin(path)
    elif is_config: descriptor_path = _generate_config(path)
    else: raise RuntimeError("invalid path provided for generation")

    # sets the default result value (nothing is returned by default) so the value is
    # not defined (unset/invalid value)
    result = None

    # in case the build flag is active the generated descriptor file is used to build
    # a new package file for the currently associated package
    if build: result = _build(descriptor_path)

    # in the descriptor is meant to be delete (not going to be used anymore) the proper
    # file must be removed from the current file system
    if delete: os.remove(descriptor_path)

    # returns the final result value that may be undefined in case no sub operations
    # have been called for the current master operation
    return result

def _generate_plugin(path):
    # imports the json module so that it's possible
    # to generate the colony descriptor file
    import json

    # retrieves the base name of the provided path so that it
    # may be "safely" used for some of the situations
    base = os.path.basename(path)
    base_dir = os.path.dirname(path)

    # prints a debug information message about the generation
    # of descriptor process that is going to be started
    output("Generating plugin descriptor for %s" % base)

    # starts some of the temporary variables that are going to be
    # used as part of the plugin structure finding process
    plugin = None
    variables = dict()

    # executes the main plugin python file so that it's possible
    # to retrieve the plugin structure and process it, at the end
    # of the "finding iteration" the plugin should have been found
    colony.legacy.execfile(path, variables)
    for name, value in variables.items():
        if not name.endswith("Plugin"): continue
        plugin = value

    # in case no plugin structure has been found an exception is raised
    # indicating that no plugin has been found (problem situation)
    if not plugin: raise RuntimeError("no plugin found")

    # uses the typical approach to the generation of the plugin short name
    # this strategy is defined as the standard one and should be respected
    # by any plugin considered to be compliant with colony
    short_name = colony.to_underscore(plugin.__name__)[:-7]

    # initializes the loop that is going to discover the type of directory
    # structure for the current plugin (either inexistent, direct or indirect)
    mode = "indirect"
    target = None
    names = os.listdir(base_dir)
    for name in names:
        current = os.path.join(base_dir, name)
        if not os.path.isdir(current): continue
        if name in (short_name, short_name + "_c"): mode = "direct"; break
        else: target = name

    # runs the proper resources gathering strategy taking into account the type
    # of plugin directory structure that has just been found in the previous step
    if mode == "direct": resources = _gather_direct(short_name, base_dir, name)
    elif mode == "indirect": resources = _gather_indirect(short_name, base_dir, target)
    else: resources = _gather_invalid(short_name, base_dir, name)

    # prepends the current plugin file to the list of resource, this is considered
    # to be the main resource to be included in the package
    resources.insert(0, base)

    # filters the resources that have been gathered so that only the ones that
    # matter are defined in the structure and then creates the sequence of dependency
    # maps that are going to be defining the dependencies of the plugin
    resources = _fitler_resources(resources)
    dependencies = [dependency.get_map() for dependency in plugin.dependencies]

    # creates the "final" plugin definition structure with the complete set of
    # attributes of the plugin and then dumps the structure using the json serializer
    # as this is the default serialization model of the descriptor files
    structure = dict(
        type = "plugin",
        platform = "python",
        sub_platforms = plugin.platforms,
        id = plugin.id,
        name = plugin.name,
        short_name = short_name,
        description = plugin.description,
        version = plugin.version,
        author = plugin.author,
        capabilities = plugin.capabilities,
        capabilities_allowed = plugin.capabilities_allowed,
        dependencies = dependencies,
        resources = resources
    )
    structure_s = json.dumps(structure)

    # verifies if the data type of the provided structure string
    # is unicode based if that's the case encodes the structure
    # using the default encoding associated with json
    if colony.legacy.is_unicode(structure_s):
        structure_s = structure_s.encode("utf-8")

    # creates the final path of the descriptor file and writes the serialized contents
    # into the file closing it for writing at the end, note that the target file is
    # defined by convention from the plugin's short name
    descriptor_path = os.path.join(base_dir, short_name + "_plugin.json")
    descriptor_file = open(descriptor_path, "wb")
    try: descriptor_file.write(structure_s)
    finally: descriptor_file.close()

    # prints a debug message about the descriptor file that has just been generated
    # so that the user is notified about the generated file
    output("Generated plugin descriptor into %s" % descriptor_path)

    # returns the generated descriptor path to the caller method so that it may be
    # used latter for any reference operation as expected
    return descriptor_path

def _generate_config(path):
    import json

    base = os.path.basename(path)
    base_dir = os.path.dirname(path)
    parent = os.path.basename(base_dir)
    name = parent + "_" + base

    description = "Configuration package (%s)" % name
    version = "0.0.0"
    author = "John Doe <john@doe.com>"

    output("Generating config descriptor for %s" % name)

    resources = _gather_config(path)
    resources = _fitler_resources(resources)

    structure = dict(
        type = "config",
        id = name,
        name = name,
        short_name = name,
        description = description,
        version = version,
        author = author,
        resources = resources
    )
    structure_s = json.dumps(structure)

    if colony.legacy.is_unicode(structure_s):
        structure_s = structure_s.encode("utf-8")

    descriptor_path = os.path.join(path, name + "_config.json")
    descriptor_file = open(descriptor_path, "wb")
    try: descriptor_file.write(structure_s)
    finally: descriptor_file.close()

    output("Generated config descriptor into %s" % descriptor_path)

    return descriptor_path

def _build(path, short_name = True):
    # imports the json module so that it's possible
    # to parse the colony descriptor file
    import json

    # opens the descriptor file to be read in the binary
    # format and loads its json contents to be used
    file = open(path, "rb")
    try: data = file.read()
    finally: file.close()

    # decodes the "raw" data using the default json encoding
    # and then runs the proper json decoding/loading
    data = data.decode("utf-8")
    descriptor = json.loads(data)

    # retrieves the various attributes from the descriptor
    # file and uses them to infer in some properties
    type = descriptor["type"]
    id = descriptor["id"]
    resources = descriptor.get("resources", [])
    extension = EXTENSIONS.get(type, ".cpx")

    # verifies if the proper type of the current package and then
    # sets the appropriate flags taking that into account
    is_plugin = type == "plugin"
    is_config = type == "config"

    # retrieves the base name for the file and removes the
    # extension from it so that the short name for it is
    # correctly retrieved taking into account the package type
    base_name = os.path.basename(path)
    if is_plugin: base_name = colony.to_underscore(base_name)[:-12]
    elif is_config: base_name = colony.to_underscore(base_name)[:-12]
    else: base_name = colony.to_underscore(base_name)[:-5]

    # retrieves the resources directory for the resources
    # from the base directory of the json descriptor and
    # then creates the name of the file from the id
    resources_directory = os.path.dirname(path)
    name = base_name + extension if short_name else id + extension

    # opens the target zip file to be used in write
    # mode (it's going to receive the data)
    file = zipfile.ZipFile(name, "w", zipfile.ZIP_DEFLATED)

    try:
        # iterates over all the resources to be written
        # in the packing file to zip them
        for resource in resources:
            # creates the full path to the resource from the
            # resources directory and the re-calculates the
            # the resources path with a prefix and writes
            # the resource into the target file
            _resource = os.path.join(resources_directory, resource)
            _relative = "resources/" + resource
            file.write(_resource, _relative)

        # writes the specification file into the packing file
        # to be used as meta data information
        file.write(path, "spec.json")
    finally:
        # closes the file to avoid any leak of file
        # descriptors and to flush the pending data
        file.close()

    # returns the generated package name to the caller method
    # as this is the resulting object for the operation
    return name

def _deploy(path, timestamp = None):
    import json

    # retrieves the currently working directory as this is going
    # to be used for some of the path resolution processes
    cwd = os.getcwd()

    # creates a new temporary directory path where the contents of
    # the package file are going to be extracted
    temp_path = tempfile.mkdtemp()

    # reads the package (zip file) and then extracts the complete
    # set of it's contents into the temporary directory so that they
    # may be manipulated and then properly used
    file = zipfile.ZipFile(path, "r", zipfile.ZIP_DEFLATED)
    try: file.extractall(temp_path)
    finally: file.close()

    # retrieves the path of the specification file and reads it's json
    # contents so that it's possible to retrieve more information about
    # the package that is currently being deployed
    spec_path = os.path.join(temp_path, "spec.json")
    file = open(spec_path, "rb")
    try: data = file.read()
    finally: file.close()
    os.remove(spec_path)

    # decodes the "raw" data using the default json encoding
    # and then runs the proper json decoding/loading
    data = data.decode("utf-8")
    descriptor = json.loads(data)

    # attaches the timestamp to the descriptor map in case it's been defined
    # by the passing arguments, this is required for compliance
    descriptor["timestamp"] = timestamp

    # retrieves the proper type from the descriptor and uses it to calculate
    # both the target value and the suffix that are going to be used in the
    # deployment operation to be performed (as expected)
    type = descriptor.get("type", "plugin")
    if type == "plugin": target = "plugins"; suffix = "_plugin"
    elif type == "config": target = "meta"; suffix = "_config"
    else: RuntimeError("invalid package type")

    # resolves the associated manager path and then uses it to
    # gather the path where the package is going to be deployed
    manager_path = resolve_manager(cwd)
    target_path = os.path.join(manager_path, target)

    # retrieves some of the descriptor information and uses it to create
    # the reference to the target path of the package deployment
    short_name = descriptor["short_name"]
    short_path = os.path.join(target_path, short_name + suffix)
    spec_path = os.path.join(short_path, "spec.json")
    resources_path = os.path.join(temp_path, "resources")

    # in case the target deployment path already exists (must be an upgrade)
    # it must be removed to avoid overlapping in files, this is considered
    # to be a safe operation only for upgrade operations
    if os.path.exists(short_path): shutil.rmtree(short_path)

    # moves the resources part of the package into the target path for the
    # package in the manager tree and then removes the temporary path
    shutil.move(resources_path, short_path)
    shutil.rmtree(temp_path)

    # dumps the current descriptor object for the item that is going to be
    # deployed and then writes the contents of it into the info based file
    # that is going to be used as a meta information provid3er
    descriptor_s = json.dumps(descriptor)
    is_unicode = colony.legacy.is_unicode(descriptor_s)
    if is_unicode: descriptor_s = descriptor_s.encode("utf-8")
    file = open(spec_path, "wb")
    try: file.write(descriptor_s)
    finally: file.close()

def _info(path):
    # retrieves the descriptor as a dictionary for the requested package
    # path, this should contain a valid dictionary with the information
    # read from the spec file associated with the package
    descriptor = _read(path)

    # retries the proper package type from the descriptor and uses it to
    # defined the correct method to be called for the info command
    type = descriptor.get("type", "plugin")
    if type == "plugin": _info_plugin(descriptor)
    elif type == "config": _info_config(descriptor)
    else: RuntimeError("invalid package type")

def _info_plugin(descriptor):
    output("id           := %s" % descriptor["id"])
    output("name         := %s" % descriptor["name"])
    output("short name   := %s" % descriptor["short_name"])
    output("version      := %s" % descriptor["version"])
    output("author       := %s" % descriptor["author"])
    output("description  := %s" % descriptor["description"])
    output("dependencies := %s" % ", ".join(descriptor["dependencies"]))
    output("capabilities := %s" % ", ".join(descriptor["capabilities"]))
    output("allowed      := %s" % ", ".join(descriptor["capabilities_allowed"]))

def _info_config(descriptor):
    output("id           := %s" % descriptor["id"])
    output("name         := %s" % descriptor["name"])
    output("short name   := %s" % descriptor["short_name"])
    output("version      := %s" % descriptor["version"])
    output("author       := %s" % descriptor["author"])
    output("description  := %s" % descriptor["description"])

def _install(name = None, id = None, version = None, upgrade = False):
    import appier

    # verifies if the provided version string is wildcard based and
    # for such situations invalidated the version value (sets to invalid)
    if version == "x.x.x": version = None

    # constructs the proper description string taking into account
    # if the name or the id has been provided and then prints a
    # message about the installation operation that is going to start
    description = name or id
    output("Installing package %s" % description)

    # creates the map containing the various parameters that are
    # going to be sent as part of the filtering process for the
    # remote request of package retrieval
    params = dict()
    if name: params["name"] = name
    if id: params["identifier"] = id

    # retrieves the proper repository url that is currently defined
    # then enforces the value to be a valid sequence, so that the
    # logic is defined as cycle of url based package detection
    repo_url = colony.conf("REPO_URL", REPO_URL)
    if not type(repo_url) in (list, tuple): repo_url = (("colony", repo_url),)

    # starts the variable that will hold the found package at invalid
    # so that the value is set only when a repo contains a package
    # matching the defined criteria
    package = None

    # iterates over the complete set of repositories defined in the
    # repository url value trying to find the proper package, note
    # that the package is found when at least one result is returned
    # matching the provided criteria (as defined in specification)
    for name, _repo_url in repo_url:
        url = _repo_url + "packages"
        result = appier.get(url, params = params)
        package = result[0] if result else dict()
        if not package: continue
        repo_url = _repo_url
        break

    # in case no package has been found for any of the defined repos
    # an exception must be raised indicating the problem to the user
    if not package: raise RuntimeError("package not found")

    # constructs the proper url for package information retrieval and
    # runs it so that the complete set of information (including dependencies)
    # is gathered providing the system with the complete set of options
    url = repo_url + "packages/%s/info" % package["name"]
    info = appier.get(url, params = dict(version = version))

    # verifies if the package is already installed under the current
    # system and if that's the case returns immediately as there's
    # nothing remaining to be done for such situation
    if _exists(info, upgrade = upgrade):
        output("Package %s is already installed, skipping" % description)
        return

    # runs the dependencies operation for the current package information
    # this operation should be able to install all the requirements for
    # the current package in transit (avoid package corruption)
    try: indent(); _dependencies(info, upgrade = upgrade)
    finally: unindent()

    # prints information about the starting of the package download, this
    # is required for the user to be notified about such action
    output("Downloading %s" % description)

    # creates the proper package retrieval url and runs the remote get request
    # to try to retrieve the package contents of so that they are installed
    url = repo_url + "packages/%s" % info["short_name"]
    data = appier.get(url, params = dict(version = info["version"]))

    # creates a new temporary directory for the new bundle file that is going
    # to be created and stores it under such directory (for deployment)
    temp_path = tempfile.mkdtemp()
    target_path = os.path.join(temp_path, "%s.cbx" % info["short_name"])
    file = open(target_path, "wb")
    try: file.write(data)
    finally: file.close()

    # runs the deployment process for the package bundle that has been retrieved
    # and then removes the temporary directory path, as it's no longer required
    _deploy(target_path, timestamp = info["timestamp"])
    shutil.rmtree(temp_path)

    # prints a message about the end of the installation process for the current
    # package, this will allow the user to be aware of the end of operation
    output("Finished installing %s" % description)

def _require(path, upgrade = False):
    # opens the file located at the provided path and reads the
    # complete set of contents from it, this should be a small
    # to medium size file so there should be no problems
    file = open(path, "rb")
    try: contents = file.read()
    finally: file.close()

    # decodes the provided contents using the expected encoding
    # of the require file (as defined is specification)
    contents = contents.decode("utf-8")

    # splits the lines in the current file and removes the ones
    # that are not considered valid, after that runs the install
    # operation for each one of the dependency requests
    lines = contents.split("\n")
    lines = [line.strip() for line in lines if line.strip()]
    for line in lines: _install(line, upgrade = upgrade)

def _upgrade():
    # starts the initial list of gathered plugins and config as
    # an empty list, this is going to be populated with the items
    # found for the current instance from the file system
    plugins = []
    configs = []

    # retrieves the current working directory and uses it to compute
    # the final manager path that is going to be used for discovery
    cwd = os.getcwd()
    manager_path = resolve_manager(cwd)

    # "calculates" both the path to the plugins directory and to the
    # meta information directory, both of them will be used to gather
    # the information on the current instace's deployment
    plugins_path = os.path.join(manager_path, "plugins")
    meta_path = os.path.join(manager_path, "meta")

    # iterates over the complete set of items in the plugins path in
    # order to be able to discover the currently installed plugins
    for item in os.listdir(plugins_path):
        if not item.endswith("_plugin"): continue
        plugins.append(item[:-7])

    # gather the information on the currently installed configs by
    # fetching information on the meta path (target config location)
    for item in os.listdir(meta_path):
        if not item.endswith("_config"): continue
        configs.append(item[:-7])

    # iterates over both the plugins and the configs to try to upgrade
    # the complete set of items as requested by the operation
    for plugin in plugins: _install(plugin, upgrade = True)
    for config in configs: _install(config, upgrade = True)

def _upload(path, repo = "colony", generate = True, delete = True):
    import json
    import appier

    # tries to runs the expansion of the glob for the provided path and
    # in case it expands to multiple values the same upload operation is
    # run on each of the items that compromise the expansion
    expansion = glob.glob(path)
    is_multiple = len(expansion) > 1
    if is_multiple:
        for item in expansion: _upload(
            item,
            repo = repo,
            generate = generate,
            delete = delete
        )
        return
    if not expansion: raise RuntimeError("no path found for '%s'" % path)
    path = expansion[0]

    # in case the generate flag is active the package file is generated
    # for the path and then the descriptor information dictionary is read
    # so that it's possible to properly upload the file to the repository
    if generate: path = _generate(path)
    descriptor = _read(path)

    # opens the path of the package file that is going to be uploaded
    # in order to be able to read the full contents of it, these are
    # going to be the contents to be uploaded to the repository
    file = open(path, "rb")
    try: contents = file.read()
    finally: file.close()

    # tries to retrieve the currently targeted repository info taking
    # into account both the environment and the static values
    repo_url = colony.conf("REPO_URL", REPO_URL)
    repo_username = colony.conf("REPO_USERNAME", "root")
    repo_password = colony.conf("REPO_PASSWORD", "root")

    # enforces the repository url to be defined as a sequence for better
    # handling of the logic and then retrieves the requested target
    # repository base url value, raising an exception in case it's not found
    if not type(repo_url) in (list, tuple): repo_url = (("colony", repo_url),)
    repo_url = dict(repo_url)
    repo_url = repo_url.get(repo)
    if not repo_url: raise RuntimeError("repository not found")

    # prints a message about the upload operation that is going to occur
    # so that the end user knows where the upload is going
    output("Uploading %s into %s repo" % (descriptor["short_name"], repo))

    # creates the url format, taking into account the defined url and the
    # current descriptor and then runs the upload, using a post operation
    url = repo_url + "packages"
    login_url = repo_url + "api/admin/login"
    auth = appier.post(login_url, params = dict(
        username = repo_username,
        password = repo_password
    ))
    appier.post(url, data_m = dict(
        sid = auth["sid"],
        identifier = descriptor["id"],
        name = descriptor["short_name"],
        version = descriptor["version"],
        type = descriptor["type"],
        info = json.dumps(descriptor),
        contents = ("contents", contents)
    ))

    # in case the delete flag is active the package file is delete after
    # a proper upload operation is performed (house keeping)
    if delete: os.remove(path)

def _read(path):
    import json

    # creates a new temporary directory path where the contents of
    # the package file are going to be extracted
    temp_path = tempfile.mkdtemp()

    # reads the package (zip file) and then extracts the complete
    # set of it's contents into the temporary directory so that they
    # may be manipulated and then properly used
    file = zipfile.ZipFile(path, "r", zipfile.ZIP_DEFLATED)
    try: file.extractall(temp_path)
    finally: file.close()

    # retrieves the path of the specification file and reads it's json
    # contents so that it's possible to retrieve more information about
    # the package that is going to have information printed
    spec_path = os.path.join(temp_path, "spec.json")
    file = open(spec_path, "rb")
    try: data = file.read()
    finally: file.close()
    shutil.rmtree(temp_path)

    # decodes the "raw" data using the default json encoding
    # and then runs the proper json decoding/loading
    data = data.decode("utf-8")
    descriptor = json.loads(data)

    # returns the read descriptor dictionary to the caller metho/function
    # so that it may be used to interpret the current package in action
    return descriptor

def _exists(info, upgrade = False):
    import json

    cwd = os.getcwd()
    manager_path = resolve_manager(cwd)

    type = info.get("type", "plugin")
    if type == "plugin": target = "plugins"; suffix = "_plugin"
    elif type == "config": target = "meta"; suffix = "_config"
    else: RuntimeError("invalid package type")

    target_path = os.path.join(manager_path, target)

    short_name = info["short_name"]
    short_path = os.path.join(target_path, short_name + suffix)

    if not os.path.exists(short_path): return False
    if not upgrade: return True

    spec_path = os.path.join(short_path, "spec.json")
    file = open(spec_path, "rb")
    try: data = file.read()
    finally: file.close()

    data = data.decode("utf-8")
    descriptor = json.loads(data)
    if not info["version"] == descriptor["version"]: return False
    if info["timestamp"] > descriptor["timestamp"]: return False

    return True

def _dependencies(info, upgrade = False):
    dependencies = info.get("dependencies", [])
    for dependency in dependencies:
        if dependency["type"] in ("package",): continue
        _install(
            id = dependency["id"],
            version = dependency["version"],
            upgrade = upgrade
        )

def _fitler_resources(resources, exclusion = (".pyc", ".temp", ".tmp")):
    filtered = []
    for resource in resources:
        if resource.endswith(exclusion): continue
        filtered.append(resource)
    return filtered

def _gather_config(base_path):
    valid = []
    result = []

    items = os.listdir(base_path)
    for item in items:
        path = os.path.join(base_path, item)
        if item.startswith("."): continue
        if not os.path.isdir(path): continue
        valid.append(path)

    for _valid in valid:
        for root, _dirs, files in os.walk(_valid):
            relative = os.path.relpath(root, base_path)
            relative = relative.replace("\\", "/")
            files = [relative + "/" + file for file in files]
            result.extend(files)

    return result

def _gather_direct(short_name, base_dir, name):
    result = []

    root_path = os.path.join(base_dir, name)
    exists = os.path.exists(root_path)
    if not exists: root_path = os.path.join(base_dir, name + "_c")

    for root, _dirs, files in os.walk(root_path):
        relative = os.path.relpath(root, root_path)
        relative = relative.replace("\\", "/")
        if relative == ".": relative = name
        else: relative = name + "/" + relative
        files = [relative + "/" + file for file in files]
        result.extend(files)

    return result

def _gather_indirect(short_name, base_dir, name):
    root_path = os.path.join(base_dir, name)

    target = None
    names = os.listdir(root_path)
    for _name in names:
        if short_name.endswith(_name): target = _name; break
        elif short_name.startswith(_name): target = _name; break

    if not target: return []

    result = [name + "/__init__.py"]

    base_path = os.path.join(root_path, target)
    for root, _dirs, files in os.walk(base_path):
        relative = os.path.relpath(root, root_path)
        relative = relative.replace("\\", "/")
        relative = name + "/" + relative
        files = [relative + "/" + file for file in files]
        result.extend(files)

    return result

def _gather_invalid(short_name, base_dir, name):
    output("No directory found for plugin %s" % short_name)
    return []

def _zip_directory(path, relative, file):
    # verifies that the provided path exists and is
    # a valid directory, in case it does not returns
    # immediately to avoid any problem, the provided
    # zip file is not going to contain of the data
    if not os.path.isdir(path): return

    # retrieves the list of entries for the path to
    # be compressed into the zip
    entries = os.listdir(path)

    # iterates over the list of entries to zip them
    # or enter in a new recursion level
    for entry in entries:
        # create the fill entry path by joining the entry
        # name to the (base) path of the directory and
        # then creates also the relative path
        _path = os.path.join(path, entry)
        _relative = os.path.join(relative, entry)

        # checks if the path is a directory and in such case runs
        # the recursion step, otherwise writes the file directly
        # into the "target" zip file
        if os.path.isdir(_path): _zip_directory(_path, _relative, file)
        else: file.write(_path, _relative)

def main():
    # retrieves the operation from the provided arguments
    # and retrieves the associated function to be executed
    operation = "help" if len(sys.argv) < 2 else sys.argv[1]
    _globals = globals()
    function = _globals.get(operation, None)
    if function: function()
    else: raise RuntimeError("invalid operation")

if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Hive Colony Framework
# Copyright (c) 2008-2015 Hive Solutions Lda.
#
# This file is part of Hive Colony Framework
#
# Hive Colony Framework is free software: you can redistribute it and/or modify
# it under the terms of the Apache License as published by the Apache
# Foundation, either version 2.0 of the License, or (at your option) any
# later version.
#
# Hive Colony Framework is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# Apache License for more details.
#
# You should have received a copy of the Apache License along with
# Hive Colony Framework If not, see <http://www.apache.org/licenses/>.

__author__ = "João Magalhães <joamag@hive.pt>"
""" The author(s) of the module """

__version__ = "1.0.0"
""" The version of the module """

__revision__ = "$LastChangedRevision$"
""" The revision number of the module """

__date__ = "$LastChangedDate$"
""" The last change date of the module """

__copyright__ = "Copyright (c) 2008-2015 Hive Solutions Lda."
""" The copyright for the module """

__license__ = "Apache License, Version 2.0"
""" The license for the module """

CRITICAL = 50
""" Critical logging level """

ERROR = 40
""" Error logging level """

WARNING = 30
""" Warning logging level """

INFO = 20
""" Info logging level """

DEBUG = 10
""" Debug logging level """

NOTSET = 0
""" Not set logging level """

WARN = WARNING
""" Alias to WARNING log level """

_levelNames = {
    CRITICAL : "CRITICAL",
    ERROR : "ERROR",
    WARNING : "WARNING",
    INFO : "INFO",
    DEBUG : "DEBUG",
    NOTSET : "NOTSET",
    "CRITICAL" : CRITICAL,
    "ERROR" : ERROR,
    "WARN" : WARNING,
    "WARNING" : WARNING,
    "INFO" : INFO,
    "DEBUG" : DEBUG,
    "NOTSET" : NOTSET,
}
""" The map relating the log levels with the textual representation and vice-versa """

def getLogger(name):
    """
    Returns the dummy logger for the given name.

    @type name: String
    @param name: The name of the logger to retrieve.
    @rtype: DummyLogger
    @return: The dummy logger for the gtiven name.
    """

    return DummyLogger("dummy")

def getLevelName(level):
    """
    Returns the textual representation of logging level.

    @type level: int
    @param level: The logging level to retrieve the textual representation.
    @rtype: String
    @return: The textual representation of logging level.
    """

    return _levelNames.get(level, ("Level %s" % level))

class DummyLogger(object):
    """
    The dummy logger class.
    """

    def __init__(self, name, level = NOTSET):
        """
        Constructor of the class.

        @type name: String
        @param name: The name of the logger.
        @type level: int
        @param level: The logging level for the new logger.
        """

        pass

    def setLevel(self, level):
        """
        Sets the level of the logger.

        @type level: int
        @param level: The level of the logger.
        """

        pass

    def debug(self, msg, *args, **kwargs):
        """
        Prints a debug message to the logger.

        @type msg: String
        @param msg: The message to print.
        """

        pass

    def info(self, msg, *args, **kwargs):
        """
        Prints an info message to the logger.

        @type msg: String
        @param msg: The message to print.
        """

        pass

    def warning(self, msg, *args, **kwargs):
        """
        Prints a warning message to the logger.

        @type msg: String
        @param msg: The message to print.
        """

        pass

    def error(self, msg, *args, **kwargs):
        """
        Prints an error message to the logger.

        @type msg: String
        @param msg: The message to print.
        """

        pass

    def exception(self, msg, *args):
        """
        Prints an exception message to the logger.

        @type msg: String
        @param msg: The message to print.
        """

        pass

    def critical(self, msg, *args, **kwargs):
        """
        Prints a critical message to the logger.

        @type msg: String
        @param msg: The message to print.
        """

        pass

    def addHandler(self, hdlr):
        """
        Adds an handler to the logger.

        @type hdlr: Handler
        @param hdlr: The handler to add to the logger.
        """

        pass

    def removeHandler(self, hdlr):
        """
        Removes an handler from the logger.

        @type hdlr: Handler
        @param hdlr: The handler to remove from the logger.
        """

        pass

class StreamHandler(object):
    """
    The stream handler class.
    """

    def setFormatter(self, fmt):
        """
        Sets the formatter for this handler.
        """

        self.formatter = fmt

class Formatter(object):
    """
    The formatter class.
    """

    def __init__(self, format):
        """
        Constructor of the class.

        @type format: String
        @param format: The formatter format.
        """

        pass



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Hive Colony Framework
# Copyright (c) 2008-2015 Hive Solutions Lda.
#
# This file is part of Hive Colony Framework
#
# Hive Colony Framework is free software: you can redistribute it and/or modify
# it under the terms of the Apache License as published by the Apache
# Foundation, either version 2.0 of the License, or (at your option) any
# later version.
#
# Hive Colony Framework is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# Apache License for more details.
#
# You should have received a copy of the Apache License along with
# Hive Colony Framework If not, see <http://www.apache.org/licenses/>.

__author__ = "João Magalhães <joamag@hive.pt>"
""" The author(s) of the module """

__version__ = "1.0.0"
""" The version of the module """

__revision__ = "$LastChangedRevision$"
""" The revision number of the module """

__date__ = "$LastChangedDate$"
""" The last change date of the module """

__copyright__ = "Copyright (c) 2008-2015 Hive Solutions Lda."
""" The copyright for the module """

__license__ = "Apache License, Version 2.0"
""" The license for the module """

class ColonyException(Exception):
    """
    The top level colony exception, this is the main exception
    for the complete colony infra-structure and all the other
    exceptions should inherit from this one.
    """

    def __unicode__(self):
        """
        Returns the unicode representation of the class.

        @rtype: String
        @return: The unicode representation of the class.
        """

        return self.__str__()

class PluginSystemException(ColonyException):
    """
    The plugin system exception class.
    """

    message = None
    """ The exception's message """

    def __init__(self, message):
        """
        Constructor of the class.

        @type message: String
        @param message: The message to be printed.
        """

        ColonyException.__init__(self)
        self.message = message

    def __str__(self):
        """
        Returns the string representation of the class.

        @rtype: String
        @return: The string representation of the class.
        """

        return "Plugin system exception - %s" % self.message

class PluginClassNotAvailable(PluginSystemException):
    """
    The plugin class not available class.
    """

    def __init__(self, message):
        """
        Constructor of the class.

        @type message: String
        @param message: The message to be printed.
        """

        PluginSystemException.__init__(self, message)
        self.message = message

    def __str__(self):
        """
        Returns the string representation of the class.

        @rtype: String
        @return: The string representation of the class.
        """

        return "Plugin class not available - %s" % self.message

class InvalidCommand(PluginSystemException):
    """
    The invalid command class.
    """

    def __init__(self, message):
        """
        Constructor of the class.

        @type message: String
        @param message: The message to be printed.
        """

        PluginSystemException.__init__(self, message)
        self.message = message

    def __str__(self):
        """
        Returns the string representation of the class.

        @rtype: String
        @return: The string representation of the class.
        """

        return "Invalid command - %s" % self.message

class InvalidArgument(PluginSystemException):
    """
    The invalid argument class.
    """

    def __init__(self, message):
        """
        Constructor of the class.

        @type message: String
        @param message: The message to be printed.
        """

        PluginSystemException.__init__(self, message)
        self.message = message

    def __str__(self):
        """
        Returns the string representation of the class.

        @rtype: String
        @return: The string representation of the class.
        """

        return "Invalid argument - %s" % self.message

class OperationNotComplete(PluginSystemException):
    """
    The operation not complete class.
    """

    def __init__(self, message):
        """
        Constructor of the class.

        @type message: String
        @param message: The message to be printed.
        """

        PluginSystemException.__init__(self, message)
        self.message = message

    def __str__(self):
        """
        Returns the string representation of the class.

        @rtype: String
        @return: The string representation of the class.
        """

        return "Operation not complete - %s" % self.message



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Hive Colony Framework
# Copyright (c) 2008-2015 Hive Solutions Lda.
#
# This file is part of Hive Colony Framework
#
# Hive Colony Framework is free software: you can redistribute it and/or modify
# it under the terms of the Apache License as published by the Apache
# Foundation, either version 2.0 of the License, or (at your option) any
# later version.
#
# Hive Colony Framework is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# Apache License for more details.
#
# You should have received a copy of the Apache License along with
# Hive Colony Framework If not, see <http://www.apache.org/licenses/>.

__author__ = "João Magalhães <joamag@hive.pt>"
""" The author(s) of the module """

__version__ = "1.0.0"
""" The version of the module """

__revision__ = "$LastChangedRevision$"
""" The revision number of the module """

__date__ = "$LastChangedDate$"
""" The last change date of the module """

__copyright__ = "Copyright (c) 2008-2015 Hive Solutions Lda."
""" The copyright for the module """

__license__ = "Apache License, Version 2.0"
""" The license for the module """

from . import number_util

def calculate_control_value(number):
    """
    Calculates the control value for the given
    number.

    @type number: int
    @param number: The number to calculate the control value.
    @rtype: int
    @return: The control value for the given
    number.
    """

    # calculates the number of digits in the number
    number_digits = number_util.get_number_length(number)

    # starts the accumulator
    accumulator = 0

    # iterates over the range of the number of digits
    for index in range(number_digits):
        # in case it's even, sets the multiplier as one
        # otherwise it must be odd and the multiplier
        # should be set as three
        if index % 2: multiplier = 1
        else: multiplier = 3

        # retrieves the current digit
        current_digit = number_util.get_digit(number, index)

        # increments the accumulator with the current digit
        # multiplied with the current multiplier
        accumulator += current_digit * multiplier

    # calculates the partial value from
    # the accumulator value
    partial_value = accumulator % 10

    # calculates the control value based
    # on the partial value
    control_value = 10 - partial_value

    # returns the control value
    return control_value



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Hive Colony Framework
# Copyright (c) 2008-2015 Hive Solutions Lda.
#
# This file is part of Hive Colony Framework
#
# Hive Colony Framework is free software: you can redistribute it and/or modify
# it under the terms of the Apache License as published by the Apache
# Foundation, either version 2.0 of the License, or (at your option) any
# later version.
#
# Hive Colony Framework is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# Apache License for more details.
#
# You should have received a copy of the Apache License along with
# Hive Colony Framework If not, see <http://www.apache.org/licenses/>.

__author__ = "João Magalhães <joamag@hive.pt>"
""" The author(s) of the module """

__version__ = "1.0.0"
""" The version of the module """

__revision__ = "$LastChangedRevision$"
""" The revision number of the module """

__date__ = "$LastChangedDate$"
""" The last change date of the module """

__copyright__ = "Copyright (c) 2008-2015 Hive Solutions Lda."
""" The copyright for the module """

__license__ = "Apache License, Version 2.0"
""" The license for the module """

class DataCacheMap(object):
    """
    Cache based map structure that may be used to store
    in memory data indexed to a certain name (may be used
    as file path) and to a certain (modification) timestamp.
    """

    data_map = {}
    """ The map associating the name of the entry with a
    tuple containing both the data of the entry and the
    timestamp from when it was last modified """

    def __init__(self):
        """
        Constructor of the class.
        """

        self.data_map = {}

    def get(self, name, timestamp = None):
        """
        Retrieves the data associated with the provided
        name and with a timestamp value equivalent to the
        one provided (if any is provided).

        @type name: String
        @param name: The key name to retrieve the the associated
        data from the map.
        @type timestamp: float
        @param timestamp: The optional timestamp vale to be used
        in the validation of the retrieved item.
        @rtype: Object
        @return: The object retrieved from the data cache map.
        """

        # in case the name is not present in the data map the
        # control must be returned to the caller function
        if not name in self.data_map: return None

        # retrieves and unpacks the data tuple into the data
        # and the timestamp then if the timestamp value is provided
        # validates it agains the "just" retrieved timestamp
        data, _timestamp = self.data_map[name]
        if timestamp and timestamp > _timestamp: return None

        # returns the "resolved" cached data
        return data

    def add(self, name, data, timestamp):
        """
        Adds a new cache entry to the map, the provided name
        is used as the key in the indexing process.

        The entry must contain both the data and the timestamp
        for the association to be possible.

        @type name: String
        @param name: The name to be used as key in the map.
        @type data: String
        @param data: The data string to be used in the map.
        @type timestamp: float
        @param timestamp: The timestamp to be used in the indexing
        process, for validation purposes.
        """

        # creates a new tuple containing both the data and the
        # timestamp and then sets it in the data map for the
        # name key (will be latter retrieved base on that key)
        self.data_map[name] = (data, timestamp)

    def remove(self, name):
        """
        Removes the data item with the provided name from the map.

        @type name: String
        @param name: The name of the item to be removed, this is the
        reference key to be used in the removal process.
        """

        # in case the name is not present in the data map, returns
        # immediately otherwise proceed with the removal process
        if not name in self.data_map: return
        del self.data_map[name]



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Hive Colony Framework
# Copyright (c) 2008-2015 Hive Solutions Lda.
#
# This file is part of Hive Colony Framework
#
# Hive Colony Framework is free software: you can redistribute it and/or modify
# it under the terms of the Apache License as published by the Apache
# Foundation, either version 2.0 of the License, or (at your option) any
# later version.
#
# Hive Colony Framework is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# Apache License for more details.
#
# You should have received a copy of the Apache License along with
# Hive Colony Framework If not, see <http://www.apache.org/licenses/>.

__author__ = "João Magalhães <joamag@hive.pt>"
""" The author(s) of the module """

__version__ = "1.0.0"
""" The version of the module """

__revision__ = "$LastChangedRevision$"
""" The revision number of the module """

__date__ = "$LastChangedDate$"
""" The last change date of the module """

__copyright__ = "Copyright (c) 2008-2015 Hive Solutions Lda."
""" The copyright for the module """

__license__ = "Apache License, Version 2.0"
""" The license for the module """

SIZE_UNITS_LIST = (
    "B", "KB", "MB", "GB", "TB", "PB", "EB", "ZB", "YB"
)
""" The size units list """

SIZE_UNIT_COEFFICIENT = 1024
""" The size unit coefficient """

DEFAULT_MINIMUM = 1024
""" The default minimum value """

def size_round_unit(size_value, minimum = DEFAULT_MINIMUM, space = False, depth = 0):
    """
    Rounds the size unit, returning a string representation
    of the value with a good rounding precision.
    This method should be used to round data sizing units.

    @type size_value: int
    @param size_value: The current size value (in bytes).
    @type minimum: int
    @param minimum: The minimum value to be used.
    @type space: bool
    @param space: If a space character must be used dividing
    the value from the unit symbol.
    @type depth: int
    @param depth: The current iteration depth value.
    @rtype: String
    @return: The string representation of the data size
    value in a simplified manner (unit).
    """

    # in case the current size value is
    # acceptable (less than the minimum)
    if size_value < minimum:
        # rounds the size value
        rounded_size_value = int(size_value)

        # converts the rounded size value to string
        rounded_size_value_string = str(rounded_size_value)

        # retrieves the size unit (string mode)
        size_unit = SIZE_UNITS_LIST[depth]

        # retrieves the appropriate separator based
        # on the value of the space flag
        separator = space and " " or ""

        # creates the size value string appending the rounded
        # size value string and the size unit
        size_value_string = rounded_size_value_string + separator + size_unit

        # returns the size value string
        return size_value_string
    # otherwise the value is not acceptable
    # and a new iteration must be ran
    else:
        # re-calculates the new size value
        new_size_value = size_value / SIZE_UNIT_COEFFICIENT

        # increments the depth
        new_depth = depth + 1

        # runs the size round unit again with the new values
        return size_round_unit(new_size_value, minimum, space, new_depth)



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Hive Colony Framework
# Copyright (c) 2008-2015 Hive Solutions Lda.
#
# This file is part of Hive Colony Framework
#
# Hive Colony Framework is free software: you can redistribute it and/or modify
# it under the terms of the Apache License as published by the Apache
# Foundation, either version 2.0 of the License, or (at your option) any
# later version.
#
# Hive Colony Framework is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# Apache License for more details.
#
# You should have received a copy of the Apache License along with
# Hive Colony Framework If not, see <http://www.apache.org/licenses/>.

__author__ = "João Magalhães <joamag@hive.pt>"
""" The author(s) of the module """

__version__ = "1.0.0"
""" The version of the module """

__revision__ = "$LastChangedRevision$"
""" The revision number of the module """

__date__ = "$LastChangedDate$"
""" The last change date of the module """

__copyright__ = "Copyright (c) 2008-2015 Hive Solutions Lda."
""" The copyright for the module """

__license__ = "Apache License, Version 2.0"
""" The license for the module """

import datetime

from . import list_util

DAY_VALUE = "day"
""" The day value """

HOUR_VALUE = "hour"
""" The hour value """

MINUTE_VALUE = "minute"
""" The minute value """

SECOND_VALUE = "second"
""" The second value """

SIMPLE_VALUE = "simple"
""" The simple value """

BASIC_VALUE = "basic"
""" The basic value """

EXTENDED_VALUE = "extended"
""" The extended value """

EXTENDED_SIMPLE_VALUE = "extended_simple"
""" The extended simple value """

MINIMIZE_MULTIPLE = "minimize_multiple"
""" The minimization with multiple value support """

MINIMIZE_UNIQUE = "minimize_unique"
""" The minimization with unique value support """

DEFAULT_INCLUDES = (
    DAY_VALUE,
    HOUR_VALUE,
    MINUTE_VALUE,
    SECOND_VALUE
)
""" The default includes list (tuple) """

DEFAULT_FORMAT = "%(D)d days, %(H)d hours, %(M)d minutes"
""" The default format """

FORMATS = {
    SIMPLE_VALUE : {
        DAY_VALUE : "%(D)d",
        HOUR_VALUE : "%(H)d",
        MINUTE_VALUE : "%(M)d",
        SECOND_VALUE : "%(S)d"
    },
    BASIC_VALUE : {
        DAY_VALUE : "%(D)dd",
        HOUR_VALUE : "%(H)dh",
        MINUTE_VALUE : "%(M)dm",
        SECOND_VALUE : "%(S)ds"
    },
    EXTENDED_VALUE : {
        DAY_VALUE : "%(D)d days",
        HOUR_VALUE : "%(H)d hours",
        MINUTE_VALUE : "%(M)d minutes",
        SECOND_VALUE : "%(S)d seconds"
    },
    EXTENDED_SIMPLE_VALUE : {
        DAY_VALUE : "%(D)d days",
        HOUR_VALUE : "%(H)d hours",
        MINUTE_VALUE : "%(M)d minutes",
        SECOND_VALUE : "%(S)d seconds"
    }
}
""" The formats map """

SEPARATORS = {
    SIMPLE_VALUE : ":",
    BASIC_VALUE : ", ",
    EXTENDED_VALUE : ", ",
    EXTENDED_SIMPLE_VALUE : " "
}
""" The separators map """

def format_seconds_smart(
    seconds,
    mode = SIMPLE_VALUE,
    includes = DEFAULT_INCLUDES,
    minimize = MINIMIZE_MULTIPLE
):
    """
    Formats the given seconds according to the given
    mode, includes and minimization support.
    The smart mode supports the creation of format strings
    dynamically from the pre-built modes.

    @type seconds: int
    @param seconds: The number of seconds to be formatted.
    @type mode: String
    @param mode: The formatting mode to be used.
    @type includes: List
    @param includes: The list of types to be included in the
    formatted string.
    @type minimize: String
    @param minimize: If the includes list should be minimized
    according to the number of seconds "available". The minimize
    string controls the way of minimization.
    @rtype: String
    @return: The string containing the formated seconds.
    """

    # in case the minimize flag is active
    if minimize:
        # re-processes the includes to according to the
        # amount of seconds available to be processed, the
        # minimize mode controls the way the include are going
        # to be minimized
        processed_includes = _process_includes(seconds, includes, minimize)
    # otherwise
    else:
        # sets the processed includes as the original includes
        processed_includes = includes

    # in case the processed includes
    # is invalid
    if not processed_includes:
        # sets the processed includes with the last
        # element of the original includes, in order to
        # avoid empty strings
        processed_includes = (
            includes[-1],
        )

    # creates the is first flag
    is_first = True

    # creates the format string lisr
    format_string_list = []

    # retrieves the formats for the mode
    formats = FORMATS[mode]

    # retrieves the separator for the mode
    separator = SEPARATORS[mode]

    for include in processed_includes:
        # in case it's the first
        # iteration
        if is_first:
            # unsets the is first flag
            is_first = False
        else:
            # adds the separator to the format
            # string list
            format_string_list.append(separator)

        # retrieves the format for the include
        format_include = formats[include]

        # adds the format include to the
        # format string list
        format_string_list.append(format_include)

    # creates the format string, joining the
    # format string list
    format_string = "".join(format_string_list)

    # formats the seconds according to the created
    # format string
    formatted_seconds = format_seconds(seconds, format_string)

    # returns the formatted seconds
    return formatted_seconds

def format_seconds(seconds, format_string = DEFAULT_FORMAT):
    """
    Formats the given seconds according to the given
    format string.

    @type seconds: int
    @param seconds: The number of seconds to be formatted.
    @type format_string: String
    @param format_string: The format string to be used.
    @rtype: String
    @return: The string containing the formated seconds.
    """

    # calculates the number of days from the seconds
    days = int(seconds / 86400.0)

    # calculates the days modulus from the seconds
    days_modulus = (seconds % 86400.0)

    # calculates the number of hours from the days modulus
    hours = int(days_modulus / 3600.0)

    # calculates the hours modules from the days modulus
    hours_modulus = days_modulus % 3600.0

    # calculates the number of minutes from the hours modulus
    minutes = int(hours_modulus / 60.0)

    # calculates the minutes modules from the hours modulus
    minutes_modulus = hours_modulus % 60.0

    # calculates the number of seconds from the minutes modulus
    seconds = int(minutes_modulus)

    # creates the attributes map with the various values
    attributes = {"D" : days, "H" : hours, "M" : minutes, "S" : seconds}

    # formats the string with the attributes map
    formatted_string = format_string % attributes

    # returns the formatted string
    return formatted_string

def timestamp_datetime(timestamp_string):
    """
    Converts the provided timestamp string to a datetime object.

    @type timestamp_string String.
    @param timestamp_string The timestamp value as string.
    """

    # retrieves the type of the timestamp string
    timestamp_string_type = type(timestamp_string)

    # in case the timestamp string is already
    # a date time object
    if timestamp_string_type == datetime.datetime:
        # returns the timestamp string itself
        # (no conversion required)
        return timestamp_string

    # converts the timestamp string to float
    timestamp_float = float(timestamp_string)

    # converts to a datetime object
    datetime_value = datetime.datetime.utcfromtimestamp(timestamp_float)

    # returns the converted datetime
    return datetime_value

def _process_includes(seconds, includes, minimize_mode):
    """
    Processes the includes list, retrieving the minimized
    includes list with only the minimum required includes.

    @type seconds: int
    @param seconds: The number of seconds to be processed.
    @type includes: List
    @param includes: The list of includes to be processed.
    @rtype: List
    @return: The processed includes
    @type minimize_mode: String
    @param minimize_mode: The minimize mode to be used.
    """

    # in case there are only seconds to be
    # represented
    if seconds < 60:
        # sets the valid includes as the seconds
        valid_includes = minimize_mode == MINIMIZE_MULTIPLE and (
            SECOND_VALUE,
        ) or (
            SECOND_VALUE,
        )
    # in case there are only second and minutes
    # to be represented
    elif seconds < 3600:
        # sets the valid includes as the minutes
        # and the seconds
        valid_includes = minimize_mode == MINIMIZE_MULTIPLE and (
            MINUTE_VALUE,
            SECOND_VALUE
        ) or (
            MINUTE_VALUE,
        )
    # in case there are seconds, minutes and
    # hours to be represented
    elif seconds < 86400:
        # sets the valid includes as the hours, the minutes
        # and the seconds
        valid_includes = minimize_mode == MINIMIZE_MULTIPLE and (
            HOUR_VALUE,
            MINUTE_VALUE,
            SECOND_VALUE
        ) or (
            HOUR_VALUE,
        )
    # in case everything should be represented
    else:
        # sets the valid includes as the days, the hours, the minutes
        # and the seconds
        valid_includes = minimize_mode == MINIMIZE_MULTIPLE and (
            DAY_VALUE,
            HOUR_VALUE,
            MINUTE_VALUE,
            SECOND_VALUE
        ) or (
            DAY_VALUE,
        )

    # intersects the (original) list of includes and the valid includes
    # to retrieve the processed includes
    processed_includes = list_util.list_intersect(includes, valid_includes)

    # returns the processed includes
    return processed_includes



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Hive Colony Framework
# Copyright (c) 2008-2015 Hive Solutions Lda.
#
# This file is part of Hive Colony Framework
#
# Hive Colony Framework is free software: you can redistribute it and/or modify
# it under the terms of the Apache License as published by the Apache
# Foundation, either version 2.0 of the License, or (at your option) any
# later version.
#
# Hive Colony Framework is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# Apache License for more details.
#
# You should have received a copy of the Apache License along with
# Hive Colony Framework If not, see <http://www.apache.org/licenses/>.

__author__ = "João Magalhães <joamag@hive.pt>"
""" The author(s) of the module """

__version__ = "1.0.0"
""" The version of the module """

__revision__ = "$LastChangedRevision$"
""" The revision number of the module """

__date__ = "$LastChangedDate$"
""" The last change date of the module """

__copyright__ = "Copyright (c) 2008-2015 Hive Solutions Lda."
""" The copyright for the module """

__license__ = "Apache License, Version 2.0"
""" The license for the module """

COUNTER = 0
""" The global counter value that will be used in the
pseudo unique number generation by incrementing the
number by one value per generation cycle """

MESSAGE_VALUE = "message"
""" The message operation value to be used in the
message oriented notification operations """

ACTION_VALUE = "action"
""" The action operation value that is going to be
used in action oriented notifications """

PROGRESS_VALUE = "progress"
""" The progress operation value that can be used
for "activities" for which the progress is monitored """

GLOBAL_HANDLERS_MAP = {}
""" The global handlers map to be used by default if
no specific handlers map is defined """

def unique():
    """
    Generates a new pseudo unique identifier generated in the
    most efficient way by incrementing a global counter.

    This operation should be used for observer based contexts
    and not outside it, in order to avoid possible problems.

    @rtype: int
    @return: The pseudo unique integer identifier generated by
    incrementing the global counter value.
    """

    global COUNTER
    COUNTER += 1
    return COUNTER

def notify(operation_name, handlers_map = None, *arguments, **named_arguments):
    """
    Notifies an handler defined in the given handlers map about
    the provided operation.
    The provided arguments are sent to the operation handler
    as parameters.

    @type operation_name: String
    @param operation_name: The name of the operation to call
    the handler.
    @type handlers_map: Dictionary
    @param handlers_map: The map containing the operation
    handlers.
    @rtype: bool
    @return: The result of the notification, useful for some
    types of operations.
    """

    # retrieves the appropriate handlers map, defaulting
    # to the global handlers map in case none is provided
    handlers_map = handlers_map or GLOBAL_HANDLERS_MAP

    # retrieves the operation method from the handlers
    # map (if possible)
    operation_method = handlers_map.get(operation_name, None)

    # in case the operation method was not found
    # (not possible to execute the handler), returns
    # immediately (as valid)
    if not operation_method: return True

    # checks if the operation method type is "iterable" and
    # in case it's not encapsulates the operation method
    # around a tuple value to make it "iterable"
    if not hasattr(operation_method, "__iter__"): operation_method = (operation_method,)

    # iterates over all the operation method contained in the
    # operation method sequence, to run their actions
    for _operation_method in operation_method:
        # calls the operation method with the arguments and
        # and "saves" the return value of it
        return_value = _operation_method(*arguments, **named_arguments)

    # returns the last return value representing the
    # final result for the notification
    return return_value

def message(handlers_map = None, *arguments, **named_arguments):
    """
    Shortcut method to call the handlers for the message
    operation.
    This method should be used as a short hand for the message
    operation.

    @type handlers_map: Dictionary
    @param handlers_map: The map containing the operation
    handlers.
    @rtype: bool
    @return: The result of the notification, useful for some
    types of operations.
    """

    return notify(MESSAGE_VALUE, handlers_map, *arguments, **named_arguments)

def action(handlers_map = None, *arguments, **named_arguments):
    """
    Shortcut method to call the handlers for the action
    operation.
    This method should be used as a short hand for the action
    operation.

    @type handlers_map: Dictionary
    @param handlers_map: The map containing the operation
    handlers.
    @rtype: bool
    @return: The result of the notification, useful for some
    types of operations.
    """

    return notify(ACTION_VALUE, handlers_map, *arguments, **named_arguments)

def progress(handlers_map = None, *arguments, **named_arguments):
    """
    Shortcut method to call the handlers for the progress
    operation.
    This method should be used as a short hand for the progress
    operation.

    @type handlers_map: Dictionary
    @param handlers_map: The map containing the operation
    handlers.
    @rtype: bool
    @return: The result of the notification, useful for some
    types of operations.
    """

    return notify(PROGRESS_VALUE, handlers_map, *arguments, **named_arguments)

def register_g(operation_mame, handler):
    """
    Registers for operations occurring for the provided name
    in the global system scope.

    @type operation_mame: String
    @param operation_mame: The name of the operation for which
    the registration is being done.
    @type handler: Function
    @param handler: The handler to be used in the handling operation
    for the operation with the provided name.
    """

    handlers = GLOBAL_HANDLERS_MAP.get(operation_mame, [])
    handlers.append(handler)
    GLOBAL_HANDLERS_MAP[operation_mame] = handlers

def unregister_g(operation_name, handler = None):
    """
    Unregisters from operations occurring for the provided name
    in the global system scope.

    In case the handler argument is present only the provided
    handler is removed from handling for the operation.

    @type operation_mame: String
    @param operation_mame: The name of the operation for which
    the "unregistration" is being done.
    @type handler: Function
    @param handler: The handler used in the handling operation
    for the operation with the provided name.
    """

    handlers = GLOBAL_HANDLERS_MAP.get(operation_name, [])
    if handler: handlers.remove(handler)
    else: del GLOBAL_HANDLERS_MAP[operation_name]

def notify_g(operation_name, *arguments, **named_arguments):
    """
    Notifies an handler defined in the given handlers map about
    the provided operation.
    The provided arguments are sent to the operation handler
    as parameters.

    This is the global notification and as such the global handlers
    are used for the handling.

    @type operation_name: String
    @param operation_name: The name of the operation to call
    the handler.
    @rtype: bool
    @return: The result of the notification, useful for some
    types of operations.
    """

    return notify(operation_name, None, *arguments, **named_arguments)



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Hive Colony Framework
# Copyright (c) 2008-2015 Hive Solutions Lda.
#
# This file is part of Hive Colony Framework
#
# Hive Colony Framework is free software: you can redistribute it and/or modify
# it under the terms of the Apache License as published by the Apache
# Foundation, either version 2.0 of the License, or (at your option) any
# later version.
#
# Hive Colony Framework is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# Apache License for more details.
#
# You should have received a copy of the Apache License along with
# Hive Colony Framework If not, see <http://www.apache.org/licenses/>.

__author__ = "João Magalhães <joamag@hive.pt>"
""" The author(s) of the module """

__version__ = "1.0.0"
""" The version of the module """

__revision__ = "$LastChangedRevision$"
""" The revision number of the module """

__date__ = "$LastChangedDate$"
""" The last change date of the module """

__copyright__ = "Copyright (c) 2008-2015 Hive Solutions Lda."
""" The copyright for the module """

__license__ = "Apache License, Version 2.0"
""" The license for the module """

import os
import setuptools

setuptools.setup(
    name = "colony",
    version = "1.3.18",
    author = "Hive Solutions Lda.",
    author_email = "development@hive.pt",
    description = "Colony Framework",
    license = "Apache License, Version 2.0",
    keywords = "colony plugin framework web",
    url = "http://getcolony.com",
    zip_safe = False,
    py_modules = [
        "colony_adm",
        "colony_start",
        "colony_wsgi"
    ],
    packages = [
        "colony",
        "colony.base",
        "colony.config",
        "colony.libs",
        "colony.test",
        "colony.test.base",
        "colony.test.libs"
    ],
    test_suite = "colony.test",
    package_dir = {
        "" : os.path.normpath("src")
    },
    package_data = {
        "colony" : ["res/*"]
    },
    entry_points = {
        "console_scripts" : [
            "cpm = colony_adm:main",
            "colony = colony_start:main",
            "colony_wsgi = colony_wsgi:main"
        ]
    },
    install_requires = [
        "appier"
    ],
    classifiers = [
        "Development Status :: 5 - Production/Stable",
        "Topic :: Utilities",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.6",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3.0",
        "Programming Language :: Python :: 3.1",
        "Programming Language :: Python :: 3.2",
        "Programming Language :: Python :: 3.3",
        "Programming Language :: Python :: 3.4",
        "Topic :: Internet :: WWW/HTTP",
        "Topic :: Internet :: WWW/HTTP :: Dynamic Content",
        "Topic :: Internet :: WWW/HTTP :: WSGI"
    ]
)



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Hive Colony Framework
# Copyright (c) 2008-2015 Hive Solutions Lda.
#
# This file is part of Hive Colony Framework
#
# Hive Colony Framework is free software: you can redistribute it and/or modify
# it under the terms of the Apache License as published by the Apache
# Foundation, either version 2.0 of the License, or (at your option) any
# later version.
#
# Hive Colony Framework is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# Apache License for more details.
#
# You should have received a copy of the Apache License along with
# Hive Colony Framework If not, see <http://www.apache.org/licenses/>.

__author__ = "João Magalhães <joamag@hive.pt>"
""" The author(s) of the module """

__version__ = "1.0.0"
""" The version of the module """

__revision__ = "$LastChangedRevision$"
""" The revision number of the module """

__date__ = "$LastChangedDate$"
""" The last change date of the module """

__copyright__ = "Copyright (c) 2008-2015 Hive Solutions Lda."
""" The copyright for the module """

__license__ = "Apache License, Version 2.0"
""" The license for the module """

class LazyClass(object):
    """
    Class representing a lazy loaded symbol, objects
    from this class may be used to represent a lazy
    loaded symbol to the end developer.
    """

    def __repr__(self):
        return "<lazy>"

    def __hash__(self):
        return None.__hash__()

    def __eq__(self, other):
        if other == None: return True
        return hash(self) == hash(other)

    def __ne__(self, other):
        return not self.__eq__(other)

    def __add__(self, other):
        return other

    def __radd__(self, other):
        return other

    def __nonzero__(self):
        return False

    def __len__(self):
        return 0

    def __iter__(self):
        return LazyIterator

class LazyIteratorClass(object):
    """
    Class representing an "empty" iterator to be used
    to provider iterator capabilities to the lazy
    object.

    This way it's possible to "iterate" over the globally
    available lazy object.
    """

    def __iter__(self):
        return self

    def __next__(self):
        raise StopIteration()

    def next(self):
        raise StopIteration()

def is_lazy(value):
    """
    Verifies if the provided value is a lazy loaded value
    by "looking" at its data type.

    This is a simple and quick util for file type verification
    and should be used as often as possible to provide a valid,
    simple and coherent interface for lazy verification/validation.

    @type value: Object
    @param value: The value that is going to be verified to
    be lazy loaded or not (lazy verification).
    @rtype: bool
    @return: If the provided value is a lazy loaded value or not
    according to the current specification.
    """

    return type(value) == LazyClass

# creates the global unique reference
# to the object to be used for lazy
# loaded symbols
Lazy = LazyClass()

# creates the global unique reference
# to the lazy iterator (singleton object)
LazyIterator = LazyIteratorClass()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Hive Colony Framework
# Copyright (c) 2008-2015 Hive Solutions Lda.
#
# This file is part of Hive Colony Framework
#
# Hive Colony Framework is free software: you can redistribute it and/or modify
# it under the terms of the Apache License as published by the Apache
# Foundation, either version 2.0 of the License, or (at your option) any
# later version.
#
# Hive Colony Framework is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# Apache License for more details.
#
# You should have received a copy of the Apache License along with
# Hive Colony Framework If not, see <http://www.apache.org/licenses/>.

__author__ = "João Magalhães <joamag@hive.pt>"
""" The author(s) of the module """

__version__ = "1.0.0"
""" The version of the module """

__revision__ = "$LastChangedRevision$"
""" The revision number of the module """

__date__ = "$LastChangedDate$"
""" The last change date of the module """

__copyright__ = "Copyright (c) 2008-2015 Hive Solutions Lda."
""" The copyright for the module """

__license__ = "Apache License, Version 2.0"
""" The license for the module """

import time
import threading

DEFAULT_SLEEP_STEP = 0.5
""" The default sleep step to be used in the scheduler """

class Scheduler(threading.Thread):
    """
    Class that implements a scheduler to be used
    to "call" callable objects for a provided timestamp.
    """

    sleep_step = None
    """ The amount of time to be used during a sleep iteration """

    continue_flag = False
    """ Flag controlling the execution of the scheduler """

    busy_flag = False
    """ Flag controlling the busy state of the scheduler """

    timestamp_queue = []
    """ Ordered list (queue) of timestamps for callables """

    timestamp_map = {}
    """ The map associating the timestamp with a list of callables """

    timestamp_lock = None
    """ The lock that controls the access to the timestamp structures """

    action_lock = None
    """ The lock that controls the access to the start and stop actions """

    def __init__(self, sleep_step = DEFAULT_SLEEP_STEP):
        """
        Constructor of the class.

        @type plugin: Plugin
        @param plugin: The plugin to be used.
        @type sleep_step: float
        @param sleep_step: The amount of time to be used
        during a sleep iteration.
        """

        threading.Thread.__init__(self)

        self.sleep_step = sleep_step

        self.daemon = True
        self.timestamp_queue = []
        self.timestamp_map = {}
        self.timestamp_lock = threading.RLock()
        self.action_lock = threading.RLock()

    def run(self):
        # iterates while the continue
        # flag is set
        while self.continue_flag:
            # acquires the timestamp lock
            self.timestamp_lock.acquire()

            try:
                # retrieves the current timestamp
                current_timestamp = time.time()

                # iterates over the timestamp queue
                while True:
                    # in case the timestamp queue is invalid
                    # (possibly empty)
                    if not self.timestamp_queue:
                        # breaks the loop (no more work
                        # to be processed for now)
                        break

                    # retrieves the timestamp from the
                    # timestamp queue
                    timestamp = self.timestamp_queue[0]

                    # in case the final timestamp has been
                    # reached
                    if current_timestamp < timestamp:
                        # breaks the loop (no more work
                        # to be processed for now)
                        break

                    # retrieves the callable (elements) list
                    # for the timestamp
                    callable_list = self.timestamp_map[timestamp]

                    # removes the callable list for the timestmap
                    # (done before the calling to avoid race condition)
                    del self.timestamp_map[timestamp]

                    # pops (removes first element) the timestamp
                    # from the timestamp queue (done before the
                    # calling to avoid race condition)
                    self.timestamp_queue.pop(0)

                    # sets the busy flag and releases the timestamp
                    # lock (avoids waiting for callables)
                    self.busy_flag = True
                    self.timestamp_lock.release()

                    try:
                        # iterates over all the callables to call
                        # them (calls the proper function)
                        for callable in callable_list:
                            # calls the callable (element)
                            # this can be of long duration
                            callable()
                    finally:
                        # acquires the timestamp lock (back)
                        # and unsets the busy flag
                        self.timestamp_lock.acquire()
                        self.busy_flag = False
            finally:
                # releases the timestamp lock
                self.timestamp_lock.release()

            # sleeps for the amount of time defined
            # in the sleep step
            time.sleep(self.sleep_step)

    def start_scheduler(self):
        """
        Starts the scheduler process.
        This method is asynchronous and the starting of
        the scheduler is not immediate.
        This method creates a new thread for scheduling.
        """

        # if the scheduler is already running avoids
        # duplicate starting, returns immediately
        if self.continue_flag: return

        # sets the continue flag
        self.continue_flag = True

        # starts the thread
        self.start()

    def stop_scheduler(self):
        """
        Stops the scheduler process.
        This method is asynchronous and the stopping of
        the scheduler is not immediate.
        """

        # if the scheduler is already stopped avoids
        # duplicate stopping, returns immediately
        if not self.continue_flag: return

        # unsets the continue flag
        self.continue_flag = False

    def reset_scheduler(self):
        """
        Resets the scheduler to the original state.
        This method may be used to avoid the allocation
        of new scheduler objects.
        This operation is dangerous an is not thread safe.
        A typical usage of this method involves first the
        stopping of the scheduler.
        """

        self.continue_flag = False
        self.timestamp_queue = []
        self.timestamp_map = {}
        self.timestamp_lock = threading.RLock()

    def add_callable(self, callbable, timestamp):
        """
        Adds a callable object to the scheduler
        for calling upon the given timestamp value.
        The sent callable is called without any arguments
        and the real time for calling may not be assured.

        @type callbable: Callable
        @param callbable: The callable object to be called
        upon in time described in the given timestamp.
        @type timestamp: float
        @param timestamp: The timestamp describing the
        time for calling the callable object.
        """

        # acquires the timestamp lock
        self.timestamp_lock.acquire()

        try:
            # starts the index value
            index = 0

            # iterates over all the timestamps in the
            # timestamp queue (to find position for insertion)
            for _timestamp in self.timestamp_queue:
                # in case the the current iteration
                # tiemstamp contains a value smaller than
                # the timestamp to be inserted
                if timestamp < _timestamp:
                    # breaks the loop (position for
                    # insertion reached)
                    break

                # increments the index
                index += 1

            # checks if the timestamp already exists in the
            # current structures
            timestamp_exists = timestamp in self.timestamp_map

            # inserts the timestamp in the timestamp queue
            # for the correct index (in order to maintain order)
            # in case it does not exist already
            not timestamp_exists and self.timestamp_queue.insert(index, timestamp)

            # retrieves the list of callabled for the given timestamp
            # and then updates it with the given callable object
            callable_list = self.timestamp_map.get(timestamp, [])
            callable_list.append(callbable)
            self.timestamp_map[timestamp] = callable_list
        finally:
            # releases the timestamp lock
            self.timestamp_lock.release()

    def is_busy(self):
        """
        Checks if the scheduler is currently in a busy status
        meaning that it's executing some sort of work.

        @rtype: bool
        @return: If the scheduler is executing any kind of work.
        """

        return self.busy_flag

    def is_running(self):
        """
        Checks if the scheduler is currently running, the scheduler
        is considered to be running if the continue flag is set.

        @rtype: bool
        @return: If the scheduler is currently running.
        """

        return self.continue_flag



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Hive Colony Framework
# Copyright (c) 2008-2015 Hive Solutions Lda.
#
# This file is part of Hive Colony Framework
#
# Hive Colony Framework is free software: you can redistribute it and/or modify
# it under the terms of the Apache License as published by the Apache
# Foundation, either version 2.0 of the License, or (at your option) any
# later version.
#
# Hive Colony Framework is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# Apache License for more details.
#
# You should have received a copy of the Apache License along with
# Hive Colony Framework If not, see <http://www.apache.org/licenses/>.

__author__ = "João Magalhães <joamag@hive.pt>"
""" The author(s) of the module """

__version__ = "1.0.0"
""" The version of the module """

__revision__ = "$LastChangedRevision$"
""" The revision number of the module """

__date__ = "$LastChangedDate$"
""" The last change date of the module """

__copyright__ = "Copyright (c) 2008-2015 Hive Solutions Lda."
""" The copyright for the module """

__license__ = "Apache License, Version 2.0"
""" The license for the module """

import colony

class BarcodeTest(colony.ColonyTestCase):
    """
    Class that tests the barcode generation methods.
    The series of tests should include the complete
    set of the barcode generation algorithms.
    """

    def test_2_to_5(self):
        """
        Tests the 2 to 5 barcode generation algorithm.
        """

        # encodes a "normal" even length based string value and asserts
        # that the encoded value is the expected one
        encoded_value = colony.encode_2_of_5("123456")
        self.assertEqual(encoded_value, "NnNnWnNwNnNnWwWnWnNwNnNwWnNwWwNnNnWnN")

        # encodes an odd length base string value and asserts that the
        # encoded value is the expected one (padded with the zero value)
        encoded_value = colony.encode_2_of_5("54321")
        self.assertEqual(encoded_value, "NnNnNwNnWwWnNnNwNwWnNnWnNwWnNnNnWwWnN")

    def test_code_128(self):
        """
        Tests the code 128 barcode generation algorithm.
        """

        # encodes a "normal" even length based string value and asserts
        # that the encoded value is the expected one
        encoded_value = colony.encode_code_128("123456")
        self.assertEqual(
            encoded_value,
            colony.legacy.u("\xcb123456/\xce", encoding = "unicode_escape")
        )

        # encodes a "normal" even length based string value and asserts
        # that the encoded value is the expected one, the used code set
        # is the a code set
        encoded_value = colony.encode_code_128("123456", "A")
        self.assertEqual(
            encoded_value,
            colony.legacy.u("\xcb123456/\xce", encoding = "unicode_escape")
        )

        # encodes a "normal" even length based string value and asserts
        # that the encoded value is the expected one, the used code set
        # is the b code set
        encoded_value = colony.encode_code_128("123456", "B")
        self.assertEqual(
            encoded_value,
            colony.legacy.u("\xcc1234560\xce", encoding = "unicode_escape")
        )

        # encodes a "normal" even length based string value and asserts
        # that the encoded value is the expected one, the used code set
        # is the c code set
        encoded_value = colony.encode_code_128("123456", "C")
        self.assertEqual(
            encoded_value,
            colony.legacy.u("\xcd,BXL\xce", encoding = "unicode_escape")
        )

    def test_code_39(self):
        """
        Tests the code 39 barcode generation algorithm.
        """

        # encodes a "normal" even length based string value and asserts
        # that the encoded value is the expected one
        encoded_value = colony.encode_code_39("123456")
        self.assertEqual(encoded_value, "*123456*")
